#! /usr/bin/env python
# -*- coding:Utf-8 -*-

###############################################################################
# === Palyno ===                                                              #
# Application Web pour le projet Climatic (http://climatic.inforef.be)        #                           #
#                                                                             #
# Programme Python fonctionnant sur la base du framework Cherrypy Version 2.2 #
# Nécessite également la présence des bibliothèques ReportLab, Gadfly &       #
# Python Imaging Library)                                                     #
###############################################################################
"""
===== (English version follows) =================

Copyright ASBL Inforef, Gérard Swinnen, Cécile Wastiaux, Louis Leclercq, 2007.
e-mail : inforef@skynet.be - gswinnen@pythomium.net
c.wastiaux@scarlet.be - louis.leclercq@ulg.ac.be
web : http://inforef.be - http://pythomium.net

Ce logiciel est un programme de simulation expérimentale destiné à l'éducation.

Ce logiciel est régi par la licence CeCILL soumise au droit français et
respectant les principes de diffusion des logiciels libres.
Vous pouvez utiliser, modifier et/ou redistribuer ce programme sous les
conditions de la licence CeCILL telle que diffusée par le CEA, le CNRS et
l'INRIA sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme, le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies. Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL, et que vous en avez accepté les
termes.

======= English version : =============

Copyright ASBL Inforef, Gérard Swinnen, Cécile Wastiaux, Louis Leclercq, 2007.
e-mail : inforef@skynet.be - gswinnen@pythomium.net
c.wastiaux@scarlet.be - louis.leclercq@ulg.ac.be
web : http://inforef.be - http://pythomium.net

This software is an educational simulation program. 

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

"""


########## Modules standards #################
import sys, os, shutil
#from struct import pack, unpack     # conversion chaines <> données binaires
from threading import Lock          # verrou de threads
from time import time, localtime, ctime, mktime, strptime, sleep
from mx import DateTime
from math import pi, sin, cos, exp     # encryptage et génération pdf
from random import randrange
from copy import deepcopy
import pickle
########## Modules d'extension ##############
import gadfly
import cherrypy            # espace de noms de CherryPy
import Image, ImageDraw, ImageChops, ImageEnhance, ImageFont, ImageFilter
from reportlab.lib.pagesizes import A4
from reportlab.lib.units import cm
from reportlab.pdfgen.canvas import Canvas
from reportlab.lib.styles import getSampleStyleSheet
from reportlab.platypus import SimpleDocTemplate, Paragraph, Spacer
#from reportlab.platypus.tables import Table, TableStyle

from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase.ttfonts import TTFont


import gettext

dos_CR =chr(13) +chr(10)
tupleType = type((1,))              # principaux types de données
stringType = type("")
listType = type([])
intType = type(3)
dicoType = type({})
# return codes for _before and _after Aspect methods
STOP = 0
CONTINUE = 1


def genPDF(methode, *args, **kwArgs):
    # Appel du générateur PDF (n'autorise qu'un seul thread à la fois)
    #Glob.pdfLock.acquire()
    try:
        # appel de l'une des méthodes de l'objet PDFgenerator :
        res = methode(*args, **kwArgs)
    except Exception, err:
        res ="<H3>Erreur renvoyée par le générateur PDF :</H3><BR>%s<BR><H3>"\
        "Si le problème persiste, veuillez en informer Pythomium.net</H3>" % err
    #Glob.pdfLock.release()
    return res      # contient une chaîne en cas d'erreur

def apostr(chaine):
    # Adapte les apostrophes au format requis par le langage SQL
    #if chaine: return chaine.replace("'","\\'")
    if chaine: return chaine.replace("'","''")
    else: return ""      # cas d'un objet None

def convUtf8(ch):
    # renvoie la chaîne ch (latin-1) réencodée en latin-1 si la version de
    # Reportlab utilisée l'exige :
    if Glob.reportlabVersion ==1:
        return ch.decode('utf-8').encode('latin-1')
    else:
        try:
            test =ch.decode("utf-8")
        except:
            ch =ch.decode("latin-1").encode("utf-8")
        return ch

def cacheCircul(im, box):
    # Construction d'un cache (masque) circulaire noir ou transparent
    # autour de l'image <im> (dont dimensions dans <box>) :
    # (image toute noire contenant une ellipse transparente) :
    ima =Image.new("RGBA", box, "black")     # nouvelle image
    draw = ImageDraw.Draw(ima)       # objet-interface pour y dessiner
    draw.ellipse((3, 3, box[0]-3, box[1]-3), fill=0)
    # Superposition du cache sur l'image :
    if Glob.bordureNoire:
        im.paste(ima, (0, 0), ima)
    else:
        # Variante : le cache ajouté ne sera pas noir, mais transparent :
        # a) inversion du cache => cercle blanc sur fond transparent
        ima =ImageChops.invert(ima)
        # b) combinaison du cache avec l'image des pollens :
        im =Image.composite(im, ima, ima)        
    return im    

def champMicroDetail(fieldSize, exemplaire):
    # Préparation de la séquence d'images agrandies à présenter dans un
    # champ microscopique secondaire. La fonction renvoie le code
    # javascript correspondant :    
    workDir ="./static/pollens/temp"
    n =0        # nombre d'images dans la séquence

    if exemplaire:
        if exemplaire[:6] =="crasse":
            # Générer des images de crasses floutées :
            for n in range(5):
                box =(fieldSize, fieldSize)
                imageG = Image.new("RGBA", box, Glob.cFondCh)
                im = Image.open("./static/pollens/%s.gif" % exemplaire)
                ima = im.resize(box, Image.ANTIALIAS)
                ima = contrastColor(ima, col=(110 + randrange(20))/100., flou=1)
                if n in (0,4):
                    ima = contrastColor(ima, flou=1)
                    ima = contrastColor(ima, flou=1)
                if n in (0,1,3,4):
                    ima = contrastColor(ima, flou=1)
                    ima = contrastColor(ima, flou=1)
                imageG.paste(ima, (0,0), ima)
                imageG = cacheCircul(imageG, box)
                imageG.save("./static/pollens/temp/crasse_%s.png" % (n+1))
            exemplaire ="crasse"
        else:
            # Repérer les fichiers-images agrandies correspondant
            # à l'exemplaire de pollen pointé :
            sourceFiles =os.listdir(workDir)
            for f in sourceFiles:
                if f[:len(exemplaire)] == exemplaire:
                    n += 1
    # Si aucun exemplaire n'est sélectionné, ou s'il n'existe aucune séquence
    # d'images agrandies pour ce exemplaire => image vide :
    if n ==0:
        exemplaire = "champ_vide"
        n =1
    # Recopier les images agrandies pour les mettre à disposition dans la
    # frame de droite de la clé de détermination (problème : des utilisateurs
    # différents vont interférer ici ) *************** à creuser ***********
    for i in range(1,11):    # effacer les photos précédentes éventuelles :
        try:
            #os.remove("./static/cle_pollen/pollen_%s.png" % (i))
            os.remove(".%s/pollen_%s.png" % (Glob.keyPath, i))
        except: pass
    for i in range(1, n+1):  # installer les photos actuelles :
        #shutil.copy("./static/pollens/temp/%s_%s.png" % (exemplaire, i),
                    #"./static/cle_pollen/pollen_%s.png" % (i))
        shutil.copy("./static/pollens/temp/%s_%s.png" % (exemplaire, i),
                    ".%s/pollen_%s.png" % (Glob.keyPath, i))

    # Script de préchargement et de sélection des images dans le navigateur :
    return ttt("com","champMicroDetail") % (n, exemplaire)


def champMicroGlobal():
    # Construire l'image d'un champ microscopique dans un fichier PNG,
    # et renvoyer (dans une chaîne) l'"image map" correspondante.
    
    # Instantiation d'une nouvelle image PIL "globale" :
    mode ="RGBA"                # mode RGB avec canal alpha
    sX =Glob.fieldSize1         # dimensions (pixels)
    box = (sX, sX)
    # On utilise une photo comme image de fond :
    okLabo = cherrypy.session.get("okLabo", None)
    if okLabo:
        n =randrange(3)
        imFile ="fond_champ_micro%s.jpg" % n     # image de fond = photo
    else:
        # Si l'on n'est pas passé par le labo, préparation "noire" :
        imFile ="champ_noir2.jpg"
    imFile = os.path.join(Glob.repPollens, imFile)        
    try:
        imageG = Image.open(imFile)
        imageG = imageG.resize(box, Image.ANTIALIAS)
    except:
        imageG = Image.new(mode, box, Glob.cFondCh)
    if not okLabo:
        # L'image présentée sera celle d'un fouillis de "crasses" :
        # Ajout d'un cache circulaire (masque) au résultat :
        imageG = cacheCircul(imageG, box)
        # Copie dans un fichier, pour que le navigateur puisse trouver l'image : 
        imFile = os.path.join(Glob.repPollens, "champ_micro.png")
        imageG.save(imFile)        
        # L'"imagemap" est réduite à un cercle englobant tout le champ.
        # cliquer sur cette zone tirera une "crasse" au hasard :
        cc = int(Glob.fieldSize1/2)      # rayon du champ micro.
        return '\n<area shape="circle" coords="%s,%s,%s" '\
        'href="javascript:nothing()" '\
        '''onclick="selectedPollen(event,'crasses_%s','%s')" />''' %\
        (cc, cc, cc, randrange(1,49), 0)
    
    # Numéro du champ d'observation courant :
    ch =cherrypy.session["champ"]
    # Disposition des grains de pollens dans ce champ :
    memPol =cherrypy.session["memPol"][ch]
    # Insertion des images de pollens dans l'image globale.
    microMap =[]                                    # zones de l'"image map"
    for ind, mp in enumerate(memPol):
        x, y, a, pol = mp[0], mp[1], mp[2], mp[3]   # position, angle, ...
        tn, rp = pol[0], pol[1]/2                   # taxon, rayon du grain
        im =pol[2].rotate(a, Image.NEAREST)         # image du grain
        # Si le taxon a déjà été déterminé : coloration en vert :
        if ind in cherrypy.session["detPol"][ch]:
            # Décomposition de l'image en ses 4 canaux :
            source = im.split()
            R,G,B,A = 0,1,2,3
            out =source[R].point(lambda i: i/1.3)
            source[R].paste(out)
            out =source[G].point(lambda i: i*1.3)
            source[G].paste(out)
            out =source[B].point(lambda i: i/1.3)
            source[B].paste(out)
            im =Image.merge(im.mode, source)
        # On utilise l'image elle-même comme masque d'insertion
        # (3e argument de paste()) afin de fusionner les transparences :
        imageG.paste(im, (x-rp, y-rp), im)
        # définir la zone cliquable dans l'"image map" :
        microMap.append('''\n<area shape="circle" coords="%s,%s,%s" href='''\
        '''"javascript:nothing()" onclick="selectedPollen(event,'%s','%s')" />'''\
        % (x, y, 12, pol[4], ind))      # pol[4] = nom de l'exemplaire précis

    # Tracé éventuel d'un cercle de sélection (centré sur le clic souris) :
    chPol_XY =cherrypy.session.get("chPol_XY", None)
    if chPol_XY:
        draw =ImageDraw.Draw(imageG)   # objet-interface pour dessiner
        x1, y1 =chPol_XY[0] -55 -5 -30, chPol_XY[1] -60 -25 -30    # -78 -25 -30
        x2, y2 =x1 +60, y1 +60
        draw.ellipse(((x1, y1),(x2,y2)),outline="red")

    # Retouche finale du contraste :
    #imageG =contrastColor(imageG, sha =4., con =1.3)
    #imageG =imageG.filter(ImageFilter.EDGE_ENHANCE)
    #imageG =imageG.filter(ImageFilter.DETAIL)
    # Ajout d'un cache circulaire (masque) au résultat :
    imageG = cacheCircul(imageG, box)
    # Copie dans un fichier, pour que le navigateur puisse trouver l'image : 
    imFile = os.path.join(Glob.repPollens, "champ_micro.png")
    imageG.save(imFile)
    return "".join(microMap)     # renvoi de l'<image map>

def champsMicroAgrandis(fieldSize):
    # Préparation des fichiers image contenant pour chaque taxon une liste
    # d'images (coupes optiques) d'un grain de pollen type pour ce taxon
    workDir ="./static/pollens/temp"
    if not os.path.exists(workDir):
        os.mkdir(workDir)
    destFiles =os.listdir(workDir)
    if len(destFiles) >730:
        return          # les fichiers existent déjà

    print "*** Building pollen images variants ..."
    sourceFiles =os.listdir("./static/pollens/agrandis")
    box =(fieldSize, fieldSize)
    for fi in sourceFiles:
        if os.path.isdir(fi): continue
        imFile = "./static/pollens/agrandis/%s" % fi
        imageG = Image.open(imFile)
        imageG = imageG.resize(box, Image.ANTIALIAS)
        imageG = contrastColor(imageG, col=1.2)
        imageG = cacheCircul(imageG, box)
        fi = fi[:-3] +"png"     # changement d'extension
        imageG.save("%s/%s" % (workDir,fi))
        print "%s image converted." % fi
    print "Total : %s images converted in library." % len(sourceFiles)
    # Ajout d'une image de champ vide :
    box =(fieldSize, fieldSize)
    imageG = Image.new("RGBA", box, Glob.cFondCh)
    imageG = cacheCircul(imageG, box)
    imageG.save("%s/champ_vide_1.png" % workDir)

def profondeursProspectees():
    # Renvoie une liste de tuples indiquant les profondeurs déjà prospectées
    # (pollens comptés) ainsi que leur datation éventuelle (ou zéro)
    rUsr =cherrypy.session["ref_usr"]
    qy ="SELECT depth, dated, biome FROM depths WHERE r_usr =%s AND checked=1 "\
    "ORDER BY depth" % rUsr
    resReq = resRequeteBD(qy) 
    if isinstance(resReq, str): return resReq       # erreur SQL
    if not len(resReq): return
    profProsp =[]
    for d in resReq:             # resReq est une liste de tuples
        depth =int(d[0])         # profondeur mémorisée
        date =int(d[1])          # datation
        biome =d[2].strip()      # biome choisi
        profProsp.append((depth, date, biome))
    return profProsp

def prepImageCarottes(profProspDates, profond):
    # Détermination des emplacements de prélèvement dans les carottes
    # <profProspDates> = liste de tuples contenant les profondeurs prospectées
    # (c.-à-d. celles pour lesquelles il faut afficher un histogramme) et
    # leurs datations C14 éventuelles
    # <profond> = profondeur choisie par l'utilisateur
    if isinstance(profProspDates, list):
        # liste des profondeurs seules    
        profProsp =[t[0] for t in profProspDates]
    else:
        profProsp =[]
    imFile ="./static/carottes.png"
    map =['<map name="caromap">']
    im = Image.open(imFile)
    larg,haut =im.size
    draw = ImageDraw.Draw(im)                       # interface pour y dessiner
    # Pour les textes : utilisation d'une police bitmap PIL :
    font = ImageFont.load("helvB14-ISO8859-1.pil")
    pro =0
    for c in range(8):
        x = c*104 +40
        draw.text((x+27, 20), "%s" % (c*-50), font =font, fill="navy")
        draw.text((x+27, 355), "%s" % (c*-50 -50), font =font, fill="navy")
        for p in range(5):
            y = p*69 +60
            rx, ry = 12, 25
            # Zone cliquable :
            # draw.rectangle((x-rx, y-ry, x+rx, y+ry), fill="grey")
            map.append('<area shape="rect" coords="%s,%s,%s,%s"  '\
                       '''href="javascript:selProfond('%s')"/>\n''' %\
                       (x-rx, y-ry, x+rx, y+ry, pro))
            if pro == profond or pro in profProsp:
                # Cercle de prélèvement :
                r =10
                if pro == profond: coul ="red"
                else: coul ="green"
                draw.ellipse((x-r, y-r, x+r, y+r), fill =coul, outline="lightgrey")
                draw.text((x+27, y-10), "-%s" % (pro+5), font =font, fill=coul)
            if p <4:
                draw.text((x+27, y+22), "-", font =font, fill="navy")
            pro += 10
            if pro >370: break
    del draw 
    im.save("./static/carrots.png")
    map.append('</map>\n')
    return "".join(map)
    
def diagrammePollinique(profProspDates):
    # Construction d'un diagramme pollinique dans une image PNG
    # Construction de l'"image map" correspondante (boutons à droite)
    # <profProspDates> contient une liste de tuples 
    # (profondeur prospectée, datation C14)
    # (c.-à-d. les profond. pour lesquelles il faut afficher un histogramme)
    profProsp =[t[0] for t in profProspDates]   # liste des profondeurs seules
    # liste des profondeurs pour lesquelles on a effectué une datation C14 :
    profDat =[t[0] for t in profProspDates if t[1]]
    workDir ="./static/user%s" % cherrypy.session["ref_usr"]
    hLigne =18                 # Hauteur des lignes
    map =['<map name="diagmap">']
    larg, haut = 893, len(Glob.nbProf)*hLigne +160       # dimensions de l'image
    # nouvelle image, avec fond blanc semi-transparent :
    im = Image.new("RGBA", (larg, haut), (255,255,255,220))
    draw = ImageDraw.Draw(im)                       # interface pour y dessiner
    # Pour les textes : utilisation d'une police bitmap PIL :
    font = ImageFont.load("helvR10-ISO8859-1.pil")
    # Pour les textes : utilisation d'une police TrueType (TTF) :
    #font = ImageFont.truetype("arial.ttf", 12)
    # Liste destinée à gérer les indications de datations C14 :
    boxDat =[]
    # Dessin des N diagrammes correspondant à chaque taxon :
    x = 40                     # coord. x de départ du diagramme
    for n, t in enumerate(Glob.taxons):     # Parcourir tous les taxons
        ta = t[0][:3]          # taxon (nom abrégé : 3 lettres)
        if ta =="cra": continue      # on ignore les 'crasses'
        s =15                  # mémorisation de la dim. max. des bâtons
        cou = t[2]             # couleur des bâtons
        # Tracé d'une ligne verticale noire :
        draw.line((x, 10, x, len(Glob.nbProf)*hLigne), fill ="black")
        # Tracé des bâtons d'histogramme :
        y =10                  # coord. verticale du bâton
        for p in Glob.nbProf:  # parcourir toutes les profondeurs
            if n ==0:          # pour le 1er taxon seulement :
                # Indication de la profondeur, à gauche :
                ts =draw.textsize(str(p), font=font)    # dim. en pixels
                draw.text((28-ts[0], y-4), str(p+5), font=font, fill="black")
                # Tracé d'une ligne horizontale grise :
                if (p/10) % 5 ==0:
                    draw.line((32,y+5,larg-20,y+5), fill="LightGrey")
                # Tracé d'une zone cliquable à droite :
                if p in profProsp:
                    draw.rectangle((811,y-2,884,y+12), fill="black")
                    draw.rectangle((810,y-3,883,y+11), fill="LightGrey")
                    draw.text((815, y-5), _("Paysage"),
                              font=font, fill="black")                    
                    map.append('<area shape="rect" coords="%s,%s,%s,%s" '\
                    '''href="javascript:selProfond('%s')"/>\n''' %\
                    (810, y-3, 883, y+11, p))
                # préparer les ajouts de datations C14 (rectangles) :
                if p in profDat:
                    date =Glob.diProfDat[p][0]
                    boxDat.append((60, y-5, 143, y+13, date))
            prop =Glob.diProfTax.get((p, ta), 0)        # proportion (%)
            if prop >s:
                s =prop    # mémoriser la taille du bâton le + grand
            # Ne dessiner que pour la(les) profondeur(s) prospectée(s) :
            if p in profProsp:
                x1, y1, x2, y2 = x, y, x+prop, y+8
                if prop >0 and prop <3:
                    x1 -=2
                    draw.ellipse((x1,y1,x2,y2), fill =cou, outline ="black")
                else:
                    draw.rectangle((x1,y1,x2,y2), fill =cou, outline ="black")
            y +=hLigne
        # Indication du taxon abrégé :
        #draw.text((x-6, y), ta.capitalize(), font=font, fill="black")
        # Indication du taxon, écrit verticalement :
        y1 =y-3
        txn =t[0]
        if ta =="eri": txn ="Ericales"
        elif ta =="ast": txn ="Cichorioidae"
        for c in txn.capitalize():
            draw.text((x-4, y1), c, font=font, fill="black")
            y1 += 12        
        # coordonnée x du bâtonnet suivant :
        x += (s +11)
    # Ajout des datations, en superposition :
    for t in boxDat:
        draw.rectangle((t[0],t[1],t[2],t[3]), fill="ivory", outline ="black")
        #txt ="%s : %s BP" % (_("Datation (C14)"), t[4])
        txt ="C14 : %s" % t[4]
        draw.text((t[0]+3, t[1]), txt, font=font, fill="black")                    
  
    del draw 
    im.save("%s/diagr_polli.png" % workDir)
    map.append('</map>\n')
    return "".join(map)
    
def histogrammePollens(prof, var=0, nfi=0):
    # Construction de l'histogramme des pollens présents à la profondeur <prof>
    # dans une image PNG. Si var =1 => l'histogramme est 'aplati'
    workDir ="./static/user%s" % (cherrypy.session["ref_usr"])
    larg =750                # dimensions de l'image
    haut =(350,100)[var]     #
    echb =(500,80)[var]      # échelle pour les bâtonnets
    yb =haut-30              # ordonnée de la ligne de base
    cFond =((245,255,230,127),(235,235,245,180))[var]
    # nouvelle image, avec fond blanc jaunâtre semi-transparent :
    im = Image.new("RGBA", (larg, haut), cFond)
    draw = ImageDraw.Draw(im)                       # interface pour y dessiner
    # Pour les textes : utilisation d'une police bitmap PIL :
    font = ImageFont.load("helvR10-ISO8859-1.pil")
    # Pour les textes : utilisation d'une police TrueType (TTF) :
    #font = ImageFont.truetype("arial.ttf", 12)
    # Calcul du coëff. de dispersion à utiliser :
    nta =0
    for t in Glob.taxons:
        ta = t[0][:3]                # taxon (nom abrégé : 3 lettres)
        if ta =="cra": continue      # on ignore les 'crasses'
        if ta =="sph": continue      # on ignore les 'sphaignes'
        prop =Glob.diProfTax.get((prof, ta), 0)        # proportion (%)
        if prop >0: nta +=1
    xd =larg/(nta+3)                 # distance entre bâtonnets (pixels)
    # Dessin des N bâtonnets correspondant à chaque taxon :
    x, hbp, tsp, dAH = 10, 0, 0, {}
    draw.line((8, yb+1, larg-8, yb+1), fill ="black")
    for typ in ("a","h"):      # traiter les arbres puis les herbes
        for n, t in enumerate(Glob.taxons):     # Parcourir tous les taxons
            taxon = t[0]
            ta =taxon[:3]                # taxon abrégé (3 car)
            if ta =="cra": continue      # on ignore les 'crasses'
            if t[3] != typ: continue     # on ignore ce qui est d'autre type
            if ta =="che": taxon ="Chenopodiaceae"   # (inutile si nom abrégé)
            elif ta =="ast": taxon ="Cichorioidae"
            elif ta =="eri": taxon ="Ericales"
            if xd <55:
                taxon =taxon[:5]      # abréger les noms pour éviter chevauch_t
            taxon =taxon.capitalize()                # Formatage du nom affiché
            prop =Glob.diProfTax.get((prof, ta), 0)  # proportion (%)
            if prop ==0: continue        # on ignore les quantités nulles
            dAH[typ] = dAH.get(typ, 0) + prop        # totalisation AP & NAP
            # Couleur du bâtonnet :
            coul ={"a":"green", "h":"GreenYellow"}[typ]
            #coul =t[2]                  # toutes coul. #, sans signification
            hb =echb*(1-exp(-prop/100.))  # hauteur (échelle non linéaire)
            draw.rectangle((x,yb,x+30,yb-hb), fill=coul)
            ts = draw.textsize(taxon, font=font)[0]
            if hb == hbp and (ts >55 or tsp >55):
                hb +=15                  # éviter le chevauchement des noms
            draw.text((x+15-ts/2, yb-hb-20), taxon, font=font, fill="black")
            draw.text((x+2, yb+8), "%s%%" % (prop), font=font, fill="black")
            x += xd
            hbp, tsp = hb, ts
    # tracé des 2 bâtonnets totalisant les arbres et herbes :
    x += (xd-15)
    for typ in ("a","h"):      # traiter les arbres puis les herbes
        coul ={"a":"green", "h":"GreenYellow"}[typ]
        leg ={"a":"Total AP", "h":"Total NAP"}[typ]
        prop =dAH[typ]
        hb =echb*(1-exp(-prop/100.))  # hauteur (échelle non linéaire)
        draw.rectangle((x, yb, x+30, yb-hb), fill=coul)
        draw.text((x-12, yb-hb-20), leg, font=font, fill="black")
        draw.text((x+2, yb+8), "%s%%" % (prop), font=font, fill="black")
        x += 50    
    del draw 
    im.save("%s/histogr_polli%s.png" % (workDir, nfi))

def paysageVirtuel(prof, nfi=0, taxList=[], taxon =""):
    # Reconstitution d'un paysage virtuel correspondant à la végétation qui a
    # produit les pollens retrouvés à la profondeur <prof>, dans une image PNG.
    # La fonction renvoie une carte des zones cliquables correspondant aux
    # éléments de paysage insérés :

    # Répertoire spécifique de l'utilisateur courant :
    workDir ="./static/user%s" % (cherrypy.session["ref_usr"])

    larg, haut = 699, 400      # dimensions de l'image
    # Tuple descriptif du fond de paysage et des taxons à dessiner pour cette
    # profondeur (sol, ère géol., nb ta1, nb ta2, nb ta3 ...) :
    lanData = Glob.diLanDat[prof]
    lTax =Glob.diLanDat[-1]         # taxons dans l'ordre
    lQua =Glob.diLanDat[-2]         # index du ou des quadril. à remplir
    lAH  =Glob.diLanDat[-3]         # indications 'arbre' ou 'herbacée'
    # nouvelle image, avec fond bleuâtre semi-transparent :
    im = Image.new("RGBA", (larg, haut), (235,235,245,100))

    # image similaire qui contiendra les taxons 'colorisés'
    im2 =Image.new("RGBA", (larg, haut), (235,235,245,100))
    
    # Rappel des emplacements et images mémorisés :
    memPH =cherrypy.session.get("memPH",[])        # empl. des herbes
    memPA =cherrypy.session.get("memPA",[])        # empl. des arbres
    memIA =cherrypy.session.get("memIA",{})        # symboles des taxons
    memIB =cherrypy.session.get("memIB",{})        # symboles taxons colorisés
    memIC =cherrypy.session.get("memIC",{})        # symboles taxons colorisés
    
    # Importation du paysage de base. Son n° est dans le dico diLanDat :
    sol =lanData[0]
    # La couleur de base est différente si le taxon 'poaceae' a été choisi :
    poa =memIA.get("poa", None)              # choisi précédemment ?
    if poa or taxon =='poaceae': base ='b'   # choisi cette fois-ci ?
    else: base ='a'
    imFile = "./static/m_sol_%s%s.gif" % (sol, base)        
    ima = Image.open(imFile)
    ima =ima.convert("RGBA")
    # Intégration de cette image dans l'image de base :
    im.paste(ima, (0, 50), ima)

    # Importation similaire dans l'image colorisée (sol = gris) :
    imFile = "./static/m_sol_%sa.gif" % (sol)
    ima = Image.open(imFile)
    ima =ima.convert("RGBA")
    # Intégration de cette image dans l'image de base :
    im2.paste(ima, (0, 50), ima)
    
    # Importation des différents taxons présents.
    # À chaque appel de cette fonction, un taxon est ajouté au paysage, lequel
    # est cependant entièrement redessiné en insérant les nouvelles plantes
    # parmi celles dont les positions ont déjà été mémorisées auparavant. 
    # Il faut d'abord dessiner les herbacées, puis les arbres ensuite,
    # en commençant à partir du fond. Pour ce faire, il faudra procéder en
    # trois phases : calculer les emplacements, trier et afficher les herbacées,
    # trier et afficher les arbres
    
    ix,iy,ex,ey = 0,0,0,0             # dimensions des images (arbres & herbes)

    #ind =lTax.index(taxon)           # ça ne marche pas avec un tuple !
    for ind, t in enumerate(lTax):    # retrouver l'indice du taxon courant
        if ind <2: continue           # ignorer les 2 premières valeurs
        if t ==taxon: break
        
    if taxon:                         # Taxon non précisé => On passe
        proR =lanData[ind]            # nombre de symboles à afficher
        # Couleur et Quadrilatère de positionnement autorisé pour ce taxon :
        ta =taxon[:3].lower()         # taxon abrégé (3 car)
        if ta =="cic": ta ="ast"
        taPr =memIA.get(ta, None)     # taxon déja traité précédemment ? 
        
        # Coordonnées du 'quadrilatère autorisé' pour placer ce taxon : 
        for t in Glob.taxons:
            if t[0][:3] ==ta:
                coul =t[2]
                q = t[4]
                if len(q) ==8:
                    # q contient les coordonnées du seul quadril. autorisé : 
                    xa,ya,xb,yb,xc,yc,xd,yd = q
                else:
                    # q contient un tuple de 5 tuples de coordonnées : 
                    iq ={1:0, 2:0, 3:0, 4:1, 5:2, 6:3, 7:4}[sol]
                    # choisir le bon :
                    xa,ya,xb,yb,xc,yc,xd,yd = q[iq]
                break

        # Préparation des images de taxons à insérer :
        imFile ="./static/m_%s.gif" % (taxon)
        ima =Image.open(imFile)
        ix, iy =ima.size
        ima =ima.convert("RGBA")
        memIA[ta] =ima           # mémorisation de l'image dans un dico
        # Même travail pour l'image de paysage 'colorisée en biôme' :
        imFile ="./static/m_%s_c.gif" % (taxon)
        ima =Image.open(imFile)
        ix, iy =ima.size
        ima =ima.convert("RGBA")
        memIB[ta] =ima           # mémorisation de l'image dans un dico
        # Pour certains taxons il faut prévoir deux colorisations différentes :
        #if ta in ("bet","fra","que","art","che"):
        if ta =="bet":
            imFile ="./static/m_%s_d.gif" % (taxon)
            ima =Image.open(imFile)
            ix, iy =ima.size
            ima =ima.convert("RGBA")
            memIC[ta] =ima           # mémorisation de l'image dans un dico
        
        # Calcul de nouvelles positions pour ce taxon :
        if lAH[ind] =="a":       # cas des arbres
            memP, dx, dy = memPA, ix/2, iy/2       
        else:                    # cas des pl. herbacées            
            memP, dx, dy = memPH, ix/1.2, iy/1.2           
        if not taPr:             # taxon déjà présent ? ne pas en rajouter ! 
            for z in range(proR):
                # Tirage au hasard d'une position dans le quadrilatère :
                n =0
                while True:
                    x =randrange(xa,xc)
                    if x >xb: y1 = yb +1.*(x-xb)*(yc-yb)/(xc-xb)   #     B
                    else: y1 = yb +1.*(xb-x)*(ya-yb)/(xb-xa)       #  A       C
                    if x >xd: y2 = yd -1.*(x-xd)*(yd-yc)/(xc-xd)   #       D
                    else: y2 = yd -1.*(xd-x)*(yd-ya)/(xd-xa)
                    y1, y2 = int(y1), int(y2)
                    if y1 == y2: y2 +=1          # pour éviter l'intervalle nul
                    y =randrange(y1, y2)
                    # Recommencer si pos. trouvée trop proche d'une autre :
                    s =True
                    for p in memP:
                        if abs(x-p[1]) < dx and abs(y-p[0]) < dy:
                            s =False ; break
                    if s: break
                    n +=1
                    if n % 50 ==0:
                        print "!! 50 itérations !!"
                        dx /= 2
                        dy /= 2
                # Mémoriser la position choisie, ainsi que le taxon :
                memP.append((y, x, ta))      # Attention : y avant x 

        # 1e phase du dessin : trier images d'herbes en fnct. de leur coord. Y :
        memPH.sort()
        # Insérer les images dans cet ordre (de l'arrière vers l'avant):
        for p in memPH:
            ima =memIA[p[2]]
            ix, iy =ima.size
            x, y = p[1]-ix/2, p[0]-iy       # attention : y vient en premier
            im.paste(ima, (x, y+53), ima)
            # même travail dans la version 'colorisée' :
            ima =memIB[p[2]]
            im2.paste(ima, (x, y+53), ima)
        # 2e phase : trier les images d'arbres en fonction de leur coord. Y :
        memPA.sort()
        # Insérer les images dans cet ordre (de l'arrière vers l'avant):
        t2bf =True
        for p in memPA:
            ima =memIA[p[2]]
            ix, iy =ima.size
            x, y = p[1]-ix/2, p[0]-iy       # attention : coord. y en premier
            im.paste(ima, (x, y+53), ima)
            # même travail dans la version 'colorisée' :
            ima =memIB[p[2]]
            #if p[2] in ("bet","fra","que","art","che"):
            if p[2] =="bet":
                # le taxon est caractéristique de 2 biômes
                t2bf = not t2bf             # alterner les 2 symboles
                if t2bf: ima =memIC[p[2]]
            im2.paste(ima, (x, y+53), ima)
    # Mémorisation, dans le répertoire de l'utilisateur courant :
    im.save("%s/paysage%s.png" % (workDir, nfi))
    
    # Mémorisation d'images réduites dans fichiers nommés d'après profondeur :
    box =(350,200)                          # Nouvelles dimensions
    im = im.resize(box, Image.ANTIALIAS)
    im.save("%s/paysa_n_%s.png" % (workDir, prof))
    im2 = im2.resize(box, Image.ANTIALIAS)
    im2.save("%s/paysa_c_%s.png" % (workDir, prof))
    
def contrastColor(im, con=None, bri=None, sha=None, tra=None,
                  col=None, flou=None):
    im = im.convert("RGBA")
    # Amélioration du contraste :
    if con:
        enh =ImageEnhance.Contrast(im)     # objet interface
        im =enh.enhance(con)
    # Ajustage de la luminosité :
    if bri:
        enh =ImageEnhance.Brightness(im)
        im =enh.enhance(bri)
    # Ajustage de la netteté :
    if sha:    
        enh =ImageEnhance.Sharpness(im)
        im =enh.enhance(sha)
    # Effet de flou :
    if flou:
        im =im.filter(ImageFilter.BLUR)
    # Décomposition de l'image en ses 4 canaux :
    if tra or col:
        source = im.split()
        R,G,B,A = 0,1,2,3
    # Coloration en rouge :
    if col:
        # On augmente la composante rouge, et on diminue les 2 autres :
        out =source[R].point(lambda i: i*col)
        source[R].paste(out)
        out =source[G].point(lambda i: i/col)
        source[G].paste(out)
        out =source[B].point(lambda i: i/col)
        source[B].paste(out)
    # Ajustage de la transparence :
    if tra:
        # selectionner les régions claires (où la comp. rouge > 180) :
        # (la fonction lambda renvoie 255 si i >180, sinon elle renvoie 0) 
        mask =source[R].point(lambda i: i>180 and 255)
        out =source[A].point(lambda i: i/tra)
        source[A].paste(out, None, mask)
    if tra or col:
        im =Image.merge(im.mode, source)
    return im

def dataFromConfig(ll, label):
    # renvoie les items qui suivent l'item '[label]\n' dans la liste ll
    try:
        ind = ll.index("[%s]\n" % label) +1
    except:
        print "*** Item [%s] introuvable dans fichier de config. ***" % label
        return
    res =""
    while ind < len(ll):
        li = ll[ind]
        if li[0] =="[" or not li: break
        res +=li
        ind +=1
    return res[:-1]     # Enlever linefeed final

def dispositionPollens():
    # Dispose une série d'images de pollens ou de "crasses" dans un champ
    # microscopique. Renvoie une liste de tuples contenant chacun
    # les coord. x et y, l'angle de rotation et la descr. détaillée d'un
    # pollen (elle-même un tuple (taxon, taille, image, ind)).
    # Les pollens choisis sont extraits de la liste des pollens présents
    # dans l'échantillon courant (cette liste sera donc réduite d'autant).
    sX =Glob.fieldSize1
    # Préparation du positionnement semi-aléatoire des grains :
    memPol =[]                  # mémorisation des positions
    cc = int(sX/2)              # coordonnées du centre (x=y)
    r_max = sX*9/20             # distance maximum du centre

    # Extraction d'images de pollens au hasard depuis cette liste globale,
    # ré-insertion dans une image de champ microscopique, et préparation
    # de l'"image map" correspondante, laquelle permettra de sélectionner
    # l'un ou l'autre de ces grains à l'aide de la souris :
    polPres =cherrypy.session["polPres"]       # pollens présents
    
    while 1:
        npr = len(polPres)       # nb. de pollens restants dansla liste
        if not npr:
            break
        h =randrange(npr)        # tirage au hasard d'un grain de la liste
        pol =polPres[h]
        #pol =polPres.pop(h)      # liste (avec réduction de celle-ci)
        
        # <pol> est un tuple contenant :
        # 0:taxon  1:taille de l'image en pixels  2:image  3:index image
        # On choisit l'emplacement du grain au hasard dans l'image,
        # en se limitant à une zone circulaire, et en s'assurant que
        # les centres d'images soient suffisamment écartés :
        d_min = pol[1]/2.5       # distance min. entre centres des images
        nn =0
        while 1:                 # boucle de recherche d'un emplacement
            angle = randrange(360)*2*pi/360
            rayon = randrange(r_max)
            x = int(rayon*cos(angle)) + cc
            y = int(rayon*sin(angle)) + cc
            # s'il s'agit de "crasses", on peut superposer les images :
            #if pol[0] =="crasses":
            #    break
            # s'il s'agit de grains de pollens, on ne superpose pas :
            ok =True
            for p in memPol:
                if x > p[0]-d_min and x < p[0]+d_min\
                and y > p[1]-d_min and y < p[1]+d_min:
                    ok =False            # trop près d'un autre !
            if ok:
                break
            nn +=1
            if nn >50:    # abandonner si l'on ne trouve aucune place
                print "50 itérations pour ", pol[0]
                ok =False
                break
        if ok:
            # On tourne l'image d'un angle aléatoire :
            a =randrange(360)
            # On mémorise position, angle et description du pollen :
            memPol.append((x,y,a,pol))
            del(polPres[h])      # on retire ce pollen de la liste
            if len(memPol) > 16:
                break   # pas plus de 17 grains par champ
        else:
            break       # plus assez de place => champ suivant

    return memPol

def enregSuivant(table, clef):
    # Renvoie le n° du premier enreg. libre dans <table>, clé d'accès <clef>
    Glob.verrou.acquire()
    res = Glob.DB.nextNum(table, clef)
    Glob.verrou.release()
    if not res: res =1      # cas d'une table vide (res =None)
    return res

def exeRequeteBD(qy):
    # Exécute la requête SQL <qy> (renvoie une chaîne si erreur, None si OK)
    Glob.verrou.acquire()
    res = Glob.DB.exeReq(qy)
    Glob.verrou.release()
    return res

def convertirPaysageD_CSV():
    # Lecture du fichier CSV contenant le tableau des n.de symboles de taxons à
    # inclure dans le paysage reconstitué pour chaque profondeur (Hautes-Fagnes)
    ofi =open("landscape_data.csv", "r")
    diLD ={}    # dico dont les clés seront les profondeurs, et les items des
                # tuples (N°dessin sol, ère, nb ta1, nb ta2, nb ta3, ...)
                # l'item de clé =-1 contient les noms des taxons dans l'ordre
                # l'item de clé =-2 contient les réf. des quadrilatères occupés
                # l'item de clé =-3 indique arbre ou pl.herbacée
    lTax =['','']    # liste des taxons (forme abrégée)
    lQuad =['','']   # liste des quadrilatères occupés par chaque taxon
    for c, ligne in enumerate(ofi):
        ligne =ligne[:-1]         # suppression \n
        ll =ligne.split(";")
        if c<2:
            continue          # ignorer les 2 premières lignes
        elif c ==2:
            # La troisième ligne contient la liste des taxons
            for n, t in enumerate(ll):
                if n <3: continue    # ignorer les 3 premiers termes
                taxon =t.replace('"','').lower()
                #if taxon[:3] =="cic": taxon ="aster"
                #lTax.append(taxon[:3])
                lTax.append(taxon)
        elif c ==41: continue
        elif c ==42:
            # la ligne n° 42 contient les indications des quadrilatères occupés
            for n, t in enumerate(ll):
                if n <3: continue    # ignorer les 3 premiers termes
                lQuad.append(t.replace('"',''))            
        else:
            # remplir une liste tampon avec les infos de la ligne :
            li =[]
            for n, t in enumerate(ll):
                if n ==0: li.append(int(t))
                elif n ==1: li.append(t.replace('"','').decode("utf-8").encode("latin-1"))
                elif n ==2: prof = int(t)-5
                else:
                    if not t: t =0
                    li.append(int(t))
            diLD[prof] =tuple(li)        # convertir la liste en tuple
    ofi.close()
    diLD[-1] =tuple(lTax)
    diLD[-2] =tuple(lQuad)
    diLD[-3] =('','','a','a','a','a','a','a','a','a','a','a','a',
               'a','a','h','h','h','h','h','h','h','h','h')
    print diLD
    # Ré-enregistrement dans fichier standardisé :
    # Enregistrement du dico dans un fichier standardisé :
    ofi =open("dicoLandscData.txt", "w")
    pickle.dump(diLD, ofi)
    ofi.close()
    
def convertirComptageHF_CSV():
    # Lecture du fichier CSV contenant le tableau des proportions des différents
    # taxons à différentes profondeurs (Réserve des Hautes-Fagnes)
    ofi =open("comptageHF.csv", "r")
    diPropPol ={}     # dico des prop. des # pollens en fct. de la profondeur
    for c, ligne in enumerate(ofi):
        ligne =ligne[:-1]         # suppression \n
        ll =ligne.split(";")
        if c ==0:
            # La première ligne contient la liste des taxons
            lTax =[]
            for n, t in enumerate(ll):
                if n ==0: continue
                taxon =t.replace('"','').lower()
                if taxon[:3] =="cic": taxon ="aster"
                lTax.append(taxon[:3])
        else:
            # remplir le dico (profondeur, taxon) ==> pourcentage
            for n, item in enumerate(ll):
                if n==0:
                    profondeur =int(item)
                elif item !='':
                    diPropPol[(profondeur, lTax[n-1])] = int(item)
    ofi.close()            
    print lTax, len(lTax)
    print diPropPol
    # Enregistrement du dico dans un fichier standardisé :
    ofi =open("dicoPropPollens.txt", "w")
    for key,item in diPropPol.items():
        ofi.write("%s;%s;%s\n" % (key[0], key[1], item))
    ofi.close()
        
def convertirCorrectifCSV():
    # Lecture du fichier CSV contenant le tableau des corrections à fournir
    # à l'utisateur en fonction des erreurs commises en cours de détermination
    #ofi =open("msg_corr_struct.csv", "r")
    ofi =open("messages_structure.csv", "r")
    c =0
    diComCor ={}    # dictionnaire-index des commentaires de correction
    while 1:
        c +=1
        li =ofi.readline()
        if not li: break
        if c<2: continue
        ll = li.split(";")
        if c==2:
            # construire la liste des taxons dans l'ordre du tableau :
            lTax =[]
            for n,t in enumerate(ll):
                if n>1:
                    taxon =t.replace('"','').lower()
                    if taxon[:3] =="cic": taxon ="aster"
                    #print n, t, taxon
                    for tr in Glob.taxons:
                        trn =tr[0]
                        #print taxon[:3],":",trn[:3],"--",
                        if taxon[:3] == trn[:3]:
                            lTax.append(taxon[:3])
        else:
            # Construire le dico (t. fourni, t. correct) => indice commentaire
            for n,ic in enumerate(ll):
                if n>1:
                    # indices du taxon fourni et du taxon correct :
                    itf, itc = c-3, n-2
                    diComCor[(lTax[itf], lTax[itc])] = int(ic)
            # Ajouter un item pour le cas où la rép. correcte est "crasse" :
            diComCor[(lTax[itf], "cra")] =99
    ofi.close()            
    print lTax, len(lTax)
    print diComCor
    # Enregistrement du dico dans un fichier standardisé :
    ofi =open("dicoIndCommCorr.txt", "w")
    for key,item in diComCor.items():
        ofi.write("%s;%s;%s\n" % (key[0], key[1], item))
    ofi.close()
    
    # convertir le fichier des commentaires de correction en une liste de 
    # tuples (n° du msg, msg, index de renvoi à la clé):
    liComCor =[]
    ofi =open("msg_corr_texte.txt", "r")
    c=0
    while 1:
        c +=1
        li =ofi.readline()
        if not li: break
        if c<6: continue
        t0 =int(li)    
        li =ofi.readline()
        if not li: break
        t1 =li.decode("utf_8").encode("latin_1")
        li =ofi.readline()
        if not li: break
        t2 =int(li)
        print t0, t1, t2
        liComCor.append((t0, t1[8:-1], t2))
    ofi.close()
    print liComCor
    # Ré-enregistrement dans un fichier standardisé :
    ofi =open(Glob.commCorr, "w")
    for tu in liComCor:
        ofi.write("%s;%s;%s\n" % (tu[0], tu[1], tu[2]))
    # Ajouter une phrase pour le cas où le pollen pointé est une "crasse" :
    cr =_("L'objet que vous avez sélectionné n'est ni un grain de pollen ni une spore.") 
    ofi.write("%s;%s;%s\n" % (99, cr, 0))
    ofi.close()                

def importerDiComCor():
    # Importer le dictionnaire des indices menant aux comm. de correction
    Glob.indComCor ={}
    ofi =open("dicoIndCommCorr.txt", "r")
    while 1:
        li =ofi.readline()
        if not li: break
        ll =li.split(";")
        Glob.indComCor[(ll[0], ll[1])] = int(ll[2][:-1])
    ofi.close()
    #print Glob.indComCor
    # Importer le dictionnaire des commentaires de correction et des 
    # renvois vers la clé dichotomique :
    Glob.diComCor ={}
    ofi =open(Glob.commCorr, "r")
    while 1:
        li =ofi.readline()
        if not li: break

        ll =li.split(";")
        n1 =int(ll[0].strip())             # delete spaces
        n2 =ll[2].replace("\n","")         # delete EOF
        n2 =int(n2.strip())                # delete spaces
        Glob.diComCor[n1] =(ll[1], n2)
    ofi.close()
    #print Glob.diComCor

def importerDiProfTax():
    # Importer le dictionnaire des prop. des diff. taxons en fct de la profond.
    # clé = tuple (profondeur, taxon abrégé) - valeur = proportion (%)
    Glob.diProfTax ={}
    ofi =open("dicoPropPollens.txt", "r")
    Glob.nbProf =[]                    # liste des profondeurs déjà traitées
    for ligne in ofi:
        ll =ligne.split(";")
        p, t = int(ll[0]), ll[1]       # profondeur en cm, taxon abrégé
        prop = int(ll[2][:-1])         # proportion en % (élimination du \n)
        Glob.diProfTax[(p, t)] = prop
        if p not in Glob.nbProf:
            Glob.nbProf.append(p)
    ofi.close()
    Glob.nbProf.sort()
    #print Glob.diProfTax

def importerDiProfDat():
    # Importer le dico des datations C14 pour chacune des profondeurs
    Glob.diProfDat ={}
    ofi =open("datations_c14.txt", "r")
    for ligne in ofi:
        ll =ligne.split(",")
        # profondeur, datation, incertitude :
        p, d, dt = int(ll[0]), int(ll[1]), int(ll[2][:-1])
        Glob.diProfDat[p] = (d, dt)
    ofi.close()

def importerListeQuesPays():
    # Importer une liste de tuples ([profondeurs],taxon,question)
    # renvoyant à des questions en rapport avec l'analyse des paysages
    # correspondant à certaines profondeurs, et à un taxon éventuel
    Glob.listeQuesPays =[]
    ofi =open("listeQuesPays.txt", "r")
    for ligne in ofi:
        ll =ligne.split(";")
        prof =ll[2][:-1].split(",")
        prof =[int(i) for i in prof]
        ques =int(ll[0])               # N°s de questions
        taxo =ll[1]
        Glob.listeQuesPays.append((prof, taxo, ques))

    #print "Glob.listeQuesPays :", Glob.listeQuesPays
    
def importerDiLanDat():
    # Importer le dictionnaire des données correspondant à la restitution d'un
    # paysage pour chaque profondeur : les clés sont les profondeurs,
    # et les items des tuples (N°dessin sol, ère, nb ta1, nb ta2, nb ta3, ...)
    # l'item dont clé =-1 contient les noms des taxons dans l'ordre
    # l'item dont clé =-2 coinient les réf. des quadrilatères occupés
    ofi =open("dicoLandscData.txt", "r")
    Glob.diLanDat = pickle.load(ofi)
    ofi.close()
    #print Glob.diLanDat[-1]
    #print Glob.diLanDat[-2]
    #print Glob.diLanDat[0]
    #print Glob.diLanDat[370]
    
def initialisation():
    # Chargement des paramètres de base depuis le fichier palyno_init.conf
    global _     # nom de la fonction de traduction (définie ici ou ailleurs)
    
    ofi =open("palyno.ini","r")
    ll =ofi.readlines()
    ofi.close()
    Glob.dbName =dataFromConfig(ll, 'dbName')
    Glob.dbDescr =dataFromConfig(ll, 'dbDescr')
    Glob.dbPath =dataFromConfig(ll, 'dbPath')
    Glob.lang =dataFromConfig(ll, 'lang')
    Glob.httpPort =int(dataFromConfig(ll, 'port'))
    Glob.reportlabVersion =int(dataFromConfig(ll, 'reportlabVersion'))
    
    # Tester la langue utilisée, et changer en conséquence le chemin
    # menant à divers fichiers :
    if Glob.lang != "fr":
        # Remplacement autom. des chaînes francophones dans le logiciel :
        t =gettext.translation("palyno", "./translations",
                               languages =[Glob.lang])
        t.install()
        # chemin des longs textes (traduits) : 
        Glob.messages ="./translations/%s/palyno_msg.txt" % Glob.lang     
        Glob.questions ="./translations/%s/palyno_quest.txt" % Glob.lang
        Glob.commCorr ="./translations/%s/dicoCommCorr.txt" % Glob.lang
        Glob.pftFile ="/static/pft_%s.jpg" % Glob.lang
        Glob.indexFile ="/static/index_%s.jpg" % Glob.lang
        #Glob.docPath ="/static/doc_%s/index.html" % Glob.lang
        Glob.docPath ="/static/doc/sommaire_palyno_%s.htm" % Glob.lang
        if not os.path.exists(".%s" % Glob.docPath):
            Glob.docPath ="/static/doc/sommaire_palyno_fr.htm"
        Glob.keyPath ="/static/cle_pollen_%s" % Glob.lang
        if not os.path.exists(".%s" % Glob.keyPath):
            Glob.keyPath ="/static/cle_pollen"
        print "*** Language used = %s ***" % Glob.lang
    else:
        Glob.messages ="palyno_msg.txt"          # chemin des longs textes (à traduire)
        Glob.questions ="palyno_quest.txt"       # chemin des questions (à traduire)
        Glob.commCorr ="dicoCommCorr.txt"
        Glob.pftFile="/static/PFT_noms.jpg"
        Glob.indexFile ="/static/index.jpg"
        Glob.docPath ="/static/doc/sommaire_palyno_fr.htm"
        Glob.keyPath ="/static/cle_pollen"
        def _(chaine):
            return chaine
        print "*** Language used = french (default) ***"

    # Descriptif des différents pollens (taxon, taille relative, couleur
    # symbolique associée, arbre/pl.herbacée, coordonnées des quadrilatères
    # décrivant les zones à remplir avec les symboles
    # de taxons dans la reconstitution des paysages virtuels :

    Glob.taxons =[["betula", 10, "Aqua", "a",
                   ((117,133,286,21,687,157,540,290),
                    (117,133,286,21,687,157,511,290),
                    (117,133,286,21,687,157,491,283),
                    (117,133,286,21,673,153,453,270),
                    (117,133,286,21,637,146,450,270))],
                  ["pinus", 24, "Chartreuse", "a",
                   (29,100,208,5,535,117,376,247)],
                  ["juniperus", 9, "BlueViolet", "a",
                   (29,100,208,5,535,117,376,247)],
                  ["corylus", 10, "pink", "a",
                   ((117,133,286,21,687,157,540,290),
                    (117,133,286,21,687,157,511,290),
                    (117,133,286,21,624,139,444,272),
                    (117,133,286,21,630,143,448,270),
                    (117,133,286,21,637,146,450,270))],
                  ["alnus", 9, "red", "a",
                   ((480,275,656,147,665,177,534,283),
                    (433,270,657,157,662,177,527,280),
                    (404,260,591,128,678,152,494,285),
                    (386,256,621,138,676,151,453,274),
                    (398,260,605,134,640,146,450,273))],
                  ["quercus", 12, "Olive", "a",
                   (29,100,208,5,535,117,376,247)],
                  ["fraxinus", 11, "RoyalBlue", "a",
                   (29,100,208,5,535,117,376,247)],
                  ["tilia", 16, "GreenYellow", "a",
                   (29,100,208,5,535,117,376,247)],
                  ["ulmus", 13, "Khaki", "a",
                   (29,100,208,5,535,117,376,247)],
                  ["fagus", 14, "Plum", "a",
                   (23,102,203,6,427,74,248,198)],
                  ["carpinus", 13, "green", "a",
                   (29,100,208,5,535,117,376,247)],
                  ["salix", 7, "Gold", "a",
                   ((272,210,437,82,689,156,539,277),
                    (272,210,437,82,689,156,494,285),
                    (272,210,437,82,689,156,480,280),
                    (272,210,437,82,668,152,443,271),
                    (272,210,437,82,633,144,440,268))],
                  ["picea", 50, "DarkGreen", "a",
                   ((36,105,220,11,687,157,540,290),
                    (36,105,220,11,687,157,511,290),
                    (36,105,220,11,687,157,491,283),
                    (36,105,220,11,673,153,453,270),
                    (103,75,174,22,485,139,418,188))],
                  ["poaceae", 13, "Gray", "h",
                   (29,100,208,5,535,117,376,247)],
                  ["ericaceae", 15, "Purple", "h",
                   ((117,133,286,21,687,157,540,290),
                    (117,133,286,21,687,157,511,290),
                    (117,133,286,21,687,157,491,283),
                    (117,133,286,21,673,153,453,270),
                    (117,133,286,21,637,146,450,270))],
                  ["artemisia", 7, "FloralWhite", "h",
                   (29,100,208,5,535,117,376,247)],
                  ["asteraceae", 13, "Gold", "h",
                   (29,100,208,5,535,117,376,247)],
                  ["chenop", 10, "SkyBlue", "h",
                   (29,100,208,5,535,117,376,247)],
                  ["plantago", 10, "DarkKhaki", "h",
                   (29,100,208,5,535,117,376,247)],
                  ["cerealia", 16, "Yellow", "h",
                   (85,115,125,89,386,207,338,230)],
                  ["filicales", 15, "Green", "h",
                   (29,100,208,5,486,99,341,235)],
                  ["cyperaceae", 14, "Peru","h",
                   ((470,278,546,259,670,157,535,289),
                    (433,270,657,157,670,157,535,289),
                    (435,269,642,142,670,157,535,289),
                    (386,256,621,138,670,157,535,289),
                    (398,260,605,134,670,157,535,289))],
                  ["sphagnum", 10, "Navy", "z",
                   (395,301,532,260,591,288,533,367)],
                  ["crasses", 12, "black", "z"]]

    # Préparation des des popups correspondant à chaque taxon et contenant
    # pour chacun une description et un lien de sélection :
    p =[]
    for t in Glob.taxons:
        tn =t[0]
        p.append('<div id="%sBigPop" class="BigPopUp" ' % (tn))
        # la description du taxon se trouve dans un 'long texte' :
        p.append('''onclick="hidePopup('%s')">%s''' % (tn, ttt("msg", tn)))
        p.append('''<a href="javascript:selectedTaxon('%s')">''' % (tn))
        p.append(_("Cliquez ici si le grain sélectionné correspond à ce taxon"))
        p.append("</a></div>\n")
    Glob.taxPop ="".join(p)
    
    
    ## Préparation des des popups correspondant à chaque taxon et contenant
    ## pour chacun un bitmap symbole et une légende :
    #p =[]
    #for t in Glob.taxons:
        #tn =t[0]
        #ta =tn[:3]
        #p.append('<div id="%sMiniPop" class="MiniPopUp" ' % (ta))
##        p.append('''onclick="hidePopup('%s')">''' % (ta))
        #p.append('<img src="/static/m_%s.gif"><br>%s' % (tn, tn.capitalize()))
##        p.append(_(" bla bla bla"))
        #p.append("</div>\n")
    #Glob.symboPop ="".join(p)



def ttt(typ, label):
    # Renvoie le texte étiqueté <label>, recherché dans l'un des fichiers
    # annexes <palyno_msg.txt> ou <palyno_com.txt>, suivant valeur de <typ>
    fichier ={"msg":Glob.messages, "com":Glob.common,
              "quest":Glob.questions}[typ]
    try:
        fi =open(fichier,"r")
    except:
        return _("*** Fichier annexe %s introuvable ***") % (fichier)
    lab = "[*%s*]" % label        # étiquette à rechercher
    lect = False                  # flag
    txt =[]                       # éléments du texte à construire 
    try:              # pour s'assurer que le fichier sera toujours refermé
        for ligne in fi:
            if lect:
                if ligne[:5] =="#####":
                    break
                else:
                    txt.append(ligne)
            elif lab in ligne:        # étiquette trouvée ==>
                lect =True           # on mémorisera les lignes suivantes
    finally:
        fi.close()
    if len(txt):   
        txt ="".join(txt)
    else:
        txt ="*** Text [%s] not found in %s ***" % (label, fichier)
        #txt =""
    return txt

def dicoQuestions():
    # Renvoyer un dico contenant toutes les questions de base
    # (On suppose que le fichier existe : test déjà effectué ailleurs)
    fi =open(Glob.questions, "r")
    dq =dict()                    # Dico des questions 
    lect = False                  # flag
    try:              # pour s'assurer que le fichier sera toujours refermé
        for ligne in fi:
            if lect:                 # on est en train d'en lire une
                if ligne[:5] =="#####":       # fin de la question
                    if len(txt): dq[nq] ="".join(txt)
                    else: dq[nq] =""
                    lect =False
                else:
                    txt.append(ligne)
            elif ligne[:2] == "[*":  # étiquette trouvée ==>
                fn =ligne.find("*]")
                nq =int(ligne[2:fn]) # numéro  
                txt =[]              # éléments du texte à construire 
                lect =True           # on mémorisera les lignes suivantes
    finally:
        fi.close()
    return dq

def trouveQuestion(n):
    # Renvoyer une liste de tuples, chacun d'eux contenant le texte de la
    # question n + une réponse éventuellement déjà fournie.
    # (On suppose que le fichier existe : test déjà effectué ailleurs)
    fi =open(Glob.questions, "r")
    ques =""          # cas où il n'existe aucune question <n>
    lect = False      # flag
    try:              # pour s'assurer que le fichier sera toujours refermé
        for ligne in fi:
            if lect:                   # on est en train de la lire
                if ligne[:5] =="#####":       # fin de la question
                    if len(txt): ques ="".join(txt)
                    else: ques =""
                    break
                else:
                    txt.append(ligne)
            elif ligne[:2] == "[*":    # étiquette trouvée ==>
                fn =ligne.find("*]")
                nq =int(ligne[2:fn])   # numéro de la question
                if nq != n: continue
                txt =[]                # éléments du texte à construire 
                lect =True             # on mémorisera les lignes suivantes
    finally:
        fi.close()
    # Une réponse aurait-elle déjà été encodée ?
    rep = trouveReponse(n)             # liste de tuples
    renvoi =[]
    for t in rep:
        renvoi.append((ques, t[0], t[1], t[2]))    # answer, rAns, variable
    return renvoi

def trouveReponse(rQ):
    # Rechercher si une ou plusieurs réponse(s) avai(en)t déjà été fournie(s)
    # à la question rQ. Renvoie une liste de tuples (1 tuple par réponse
    # fournie), ou une chaîne si erreur
    rUsr =cherrypy.session["ref_usr"]
    qy ="SELECT r_ans, answer, variable FROM answers "\
    "WHERE r_usr =%s AND r_que =%s" % (rUsr, rQ)
    res = resRequeteBD(qy) 
    if isinstance(res, str): return res       # erreur SQL
    if len(res):     # il existait déjà une ou des réponse(s)
        env =list()
        for rep in res:
            rAns =rep[0]
            answer =rep[1].strip()
            variable =rep[2].strip()
            env.append((answer, rAns, variable))
    else:
        rAns =0
        answer =""
        variable =""
        env =[("", 0, "")]
    return env 

def memoriseReponse(rAns, answer, variable, rQue):
    # Mémorisation d'une réponse entrée :
    rUsr =cherrypy.session["ref_usr"]
    rAns =int(rAns)
    if rAns >0:        # une réponse avait déjà été fournie
        qy ="UPDATE answers SET answer ='%s' WHERE r_ans =%s" %\
        (apostr(answer), rAns)
    else:    
        rAns =Glob.DB.nextNum("answers", "r_ans")
        qy ="INSERT INTO answers (r_ans, r_usr, r_que, answer, "\
        "variable) VALUES (%s, %s, %s, '%s', '%s')" %\
        (rAns, rUsr, rQue, apostr(answer), apostr(variable))
    res =exeRequeteBD(qy)
    
    if res: return res               # error msg

def now():
    # Renvoie la date et l'heure système, en clair
    if Glob.lang != "fr": return ctime(time())
    t = localtime(time())     		# renvoie un tuple avec date et heure
    return "le %s-%s-%s à %s:%s:%s" % (t[2],t[1],t[0],t[3],t[4],t[5])

def ouvertureBD():
    # Ouverture base de données - chargement des paramètres globaux courants
    # Les verrous sont créés par la classe Lock() du module threading : 
    Glob.verrou =Lock()           # verrou  pour synchro des threads
    Glob.DB =GestionBD()                    # Création objet interface
    if not Glob.DB.connexion: return        # BD non accessible

def pollensPresents(fieldSize):
    # Renvoie une liste des pollens présents dans l'échantillon correspondant
    # à la profondeur de prélèvement courante, sous la forme de tuples
    # descriptifs de micro-objets (pollens ou "crasses").
    # Chaque tuple contient :
    # le nom du taxon, la taille de l'image (en pixels), l'image elle-même,
    # le n° d'ordre de cette image dans Glob.microDic[taxon],
    # le nom de base du fichier image correspondant à l'exemplaire précis
    pro =cherrypy.session["profond"]    # profondeur
    
    fichiers =os.listdir(Glob.repPollens)   # liste des fichiers-images
    lip =[]
    # Installer les pollens significatifs :
    for t in Glob.taxons:
        txn =t[0]                              # taxon proprement dit
        if txn =="crasses":
            continue
        tna =txn[:3]                           # alias (taxon abrégé)
        p =Glob.diProfTax.get((pro, tna), 0)   # proportion présente (%)
        nid =len(Glob.microDic[txn])           # nb. d'images disponibles
        if not nid: continue                   # ********* provisoire *********
        for u in range(p/2):                   # nombres = pourcentages / 2
            ex =randrange(nid)                 # tirage aléat. d'1 exemplaire
            im =Glob.microDic[txn][ex][0]      # image elle-même
            fi =Glob.microDic[txn][ex][1]      # nom de fichier de l'exemplaire
            # t[1] contient la taille relative du grain
            aps =fieldSize * t[1] /(45 +randrange(5))
            box =(aps, aps)                    # taille résultante (pixels)
            im = im.resize(box, Image.ANTIALIAS)
            lip.append((txn, aps, im, ex, fi))

    # Installer les "crasses" :
    for ex, tuIm in enumerate(Glob.microDic["crasses"]):
        im, fi = tuIm[0], tuIm[1]
        for u in range(randrange(3)):       # n exemplaires de chaque
            dim =im.getbbox()               # dimensions image originale
            dim =max(dim[2], dim[3])        # dimension la plus grande
            aps =fieldSize * dim / (1000 +randrange(200))
            box =(aps, aps)         # taille résultante de l'image (pixels)
            im = im.resize(box, Image.ANTIALIAS)
            lip.append(("crasses", aps, im, ex, fi))

    return lip
            
def pollensEtCrasses():
    # Construction du dictionnaire global contenant pour chaque taxon une
    # liste d'images microscopiques :
    Glob.microDic ={}
    fichiers =os.listdir(Glob.repPollens)   # liste des fichiers-images
    for t in Glob.taxons:
        tn =t[0]                            # taxon proprement dit
        rps =t[1]                           # taille relative du grain
        tn0 =tn +"_"
        Glob.microDic[tn] =[]
        for f in fichiers:
            if f[:len(tn0)] == tn0:
                imFile = os.path.join(Glob.repPollens, f)  
                imageG = Image.open(imFile)
                imageG = contrastColor(imageG, sha=1.3, tra=1.15, con =1.3,
                                col=(110 + randrange(20))/100.)

                #imageG =imageG.filter(ImageFilter.EDGE_ENHANCE)
                # recadrer l'image dans une "boîte" carrée plus grande
                # (25 pixels de plus tout autour), afin d'autoriser
                # des rotations sans pertes) :
                dim =imageG.getbbox()           # dimensions image originale
                dx, dy = dim[2], dim[3]
                pz = 25 + abs(dx-dy)/2
                if dx > dy:                     # dimension la plus grande
                    diCar =dx+50; px =25; py =pz
                else:
                    diCar =dy+50; px =pz; py =25
                box =(diCar, diCar)             # boîte carrée
                # nouvelle image transparente (canal alpha=0) :
                ima = Image.new("RGBA", box, (255,255,180,0))
                ima.paste(imageG, (px, py), imageG)

                # mémoriser l'image et son nom (sans extension) :
                Glob.microDic[tn].append((ima, f[:-4]))

        print "%s %s images stored in microDic." %\
        (len(Glob.microDic[tn]), tn)

    #print "%s taxons au total (y compris les 'crasses')" % len(Glob.microDic)
    
def resRequeteBD(qy):
    # Renvoie résultat de la requête <qy> (liste de dicos si OK, chaîne sinon)
    Glob.verrou.acquire()
    res = Glob.DB.listResult(qy)
    Glob.verrou.release()
    return res

class Glob:
    # Classe servant d'espace de noms pour les variables pseudo-globales.
    # Note : certaines variables Glob. sont définies dans la procédure
    # initialisation()   
    
    version ="1.03"
    version_date ='13/12/07'
    #lang ="fr"                          # voir initialisation()
    #reportlabVersion                   # voir initialisation()
    demoVersion =False                  # brider fonctionnalités et BD
    bdLock =0                           # réservation de l'accès aux admins
    fichJour ="palyno_log.txt"          # chemin du journal des logins
    #messages ="palyno_msg.txt"          # chemin des longs textes (dynamique)
    #questions ="palyno_quest.txt"       # chemin des questions (dynamique)
    common ="palyno_com.txt"            # chemin des longs textes techniques
    repArchives ="./bak"                # répertoire des fichiers d'échange
    repPollens ="./static/pollens"      # images (photos) des pollens
    dbName ="dummy_db"                  # nom de la base de données
    dbDescr ="dummy_descr"              # description de la bd
    dbPath ="dummy_dir"                 # répertoire contenant la BD
    bordureNoire =0                     # bordure noire pour les champs micro.
    fieldSize1 =340                     # taille du champ micro. principal
    fieldSize2 =270                     # taille du champ micro. secondaire
    cFondCh =(230,215,180,235)          # couleur de fond (R,G,B,Alpha)
    #licensedTo ="nobody"        # not yet used
    license ="license"          # not yet used
    #owner ="owner"              # not yet used
    nbProf =0               # nombre total de profondeurs envisagées
    taxons =[]              # Liste des taxons avec leurs tailles relatives
    microDic ={}            # Dico contenant les images de pollens et crasses
    diProfTax ={}           # Dico des prop. des taxons aux différ. profondeurs
                            # k: tuple (profondeur, taxon abr.) - v: prop. en %
    diProfDat ={}           # Dico des datations C14 de chaque profondeur
    indComCor ={}           # Dico des indices menant aux comm. de correction
    diComCor ={}            # Dico des commentaires de correction et des 
                            # renvois vers la clé dichotomique
    diLanDat ={}            # Dico décrivant les paysages à reconstituer
    listeQuesPays =[]       # Liste renvoyant vers des questions sur paysages
    
    

class GestionBD:
    # Mise en place et interfaçage d'une base de données Gadfly.
    
    def __init__(self):
        # Création de l'objet interface self.connexion
        try:
            self.connexion = gadfly.gadfly(Glob.dbName, Glob.dbPath)
        except Exception, err:
            print err
            # À désactiver dans version finale ? :
            # La base de données n'existe probablement pas. On la crée :
            self.creationBD()
            self.dbi_msg =_("*** Données inaccessibles ***</BR>") +\
            _("Le serveur de données renvoie l'erreur suivante :</BR>") +\
            err.message +"</BR>"
        except:            
            self.dbi_msg =_("*** Données inaccessibles ***</BR>")
        else:
            self.dbi_msg =_("Serveur de données (re)démarré %s.") % (now())
            #"</BR>Base de données active : %s." % (Glob.dbName)

    def creationBD(self):
        # Création de la base de données (elle devrait exister !) :
        try:
            #os.system("mkdir %s" % (Glob.dbPath))
            os.mkdir(Glob.dbPath)
        except:
            pass  
        self.connexion =gadfly.gadfly()
        self.connexion.startup(Glob.dbName, Glob.dbPath)
        # Structure de la base de données :
        tList =(
            ("users","Identifiants utilisateurs",
              ("r_usr","k"), ("nom",30), ("prenom",30), ("passwd",20)),
            ("depths","Profondeurs déjà prospectées",
              ("r_pro","k"), ("r_usr","i"), ("depth","i"), ("errors","i"),
              ("checked","i"), ("dated","i"), ("biome",5)),
            ("answers","Réponses aux questions",
              ("r_ans","k"), ("r_usr","i"), ("r_que","i"), ("answer",500),
              ("variable",50))
            )
        # Création de toutes les tables :
        for table in tList:
            n_t = table[0]
            print "****** Création de la table %s ******" %(n_t)
            qy = "CREATE TABLE %s (" %(n_t)
            for ch in table:
                if type(ch) == type("chaine"): continue
                n_ch = ch[0]          # nom du champ
                if ch[1] =='k' or ch[1] =='i':
                    t_ch ='INTEGER'
                else:
                    t_ch ='VARCHAR'
                qy = qy + "%s %s, " % (n_ch, t_ch)
            qy = qy[:-2] + ")"
            #print qy
            res =self.exeReq(qy)
            if type(res) == stringType: print res
        self.connexion.close()

    def exeReq(self, qy):
        # Exécute la requête <qy>
        # Renvoie None, ou un message d'erreur s'il s'en produit une.
        
        # D'abord supprimer les point-virgule éventuels dans la requête,
        # pour boucher un trou de sécurité SQL classique :
        qy =qy.replace(";"," ")

        try:
            dbi =self.connexion.cursor()
            dbi.execute(qy)
            self.connexion.commit()
        except Exception, err:
            # afficher la requête et le message d'erreur système :
            return "Requête incorrecte : </BR>%s</BR>" % (qy) +\
            "Erreur signalée par le serveur SQL :</BR>%s</BR>" % err
        except:
            # afficher la requête si autre erreur indéterm. :
            return "Impossible d'exécuter la requête : </BR>%s</BR>" % (qy)
        else:
            return None

    def listResult(self, qy):
        # Exécute la requête <qy>, et renvoie le résultat sous la forme
        # d'une liste de tuples, ou d'un message d'erreur s'il y a lieu.

        # D'abord supprimer les point-virgule éventuels dans la requête,
        # pour boucher un trou de sécurité SQL classique :
        qy =qy.replace(";"," ")

        try:
            dbi =self.connexion.cursor()
            dbi.execute(qy)
            self.resReq = dbi.fetchall()
        except Exception, err:
            # afficher la requête et le message d'erreur système :
            self.resReq ="Requête incorrecte : </BR>%s</BR>" % (qy) +\
            "Erreur signalée par le serveur SQL :</BR>%s</BR>" % err
            erDet =True
        except:
            # afficher la requête si autre erreur indéterm. :
            self.resReq ="Impossible d'exécuter la requête : </BR>%s</BR>" % (qy)
            erDet =True
        else:
            erDet =False
        if erDet:
            return self.resReq          # = message d'erreur (chaîne)
        else:
            return self.resReq          # = liste de tuples

    def nextNum(self, table, pK):
        # Renvoie le premier n° libre pour la clé primaire <pK>
        qy ="SELECT MAX(%s) FROM %s" % (pK, table)
        res = self.listResult(qy)
        if type(res) ==stringType: lastRec =0
        else: lastRec = res[0][0]
        #print "table %s clef %s max %s" % (table, pK, lastRec)
        #if lastRec >0:
        #print "res :", res, "lastRec :", lastRec, type(lastRec)
        return lastRec +1       # renvoi premier n° libre

class Page(object):
    # Modèle de page contenant l'en-tête et le pied de page communs,
    # lesquels mettent en place un logo et un titre en haut de page,
    # ainsi qu'une marge de gauche contenant quelques liens.

    def __getattribute__(self, methodName):
        # find method specified by methodName
        try:
            method = object.__getattribute__(self, methodName)
        except:
            raise

        # if requested attribute is not a method, simply return it
        if not callable(method):
            return method

        # define wrapper function
        def _wrapper(*k, **kw):
            # call _before method
            status, value = object.__getattribute__(self, '_before')(methodName, method)
            if status == STOP:
                return value

            # call wrapped method and append results
            result = method(*k, **kw)
            if value:
                result = value + result

            # call _after method
            status, value =  object.__getattribute__(self, '_after')(methodName, method)
            if status == STOP:
                return value
            if value:
                result += value

            return result

        # expose wrapper function if wrapped method is exposed
        if getattr(method, 'exposed', None):
            _wrapper.exposed = True

        # return wrapper function. It'll get called instead of the
        # requested method.
        return _wrapper


    def header(self):
        # Image utilisée comme logo en haut à gauche :
        logo = "/static/erica_1.png"
        
        # Remplissage de la barre d'outils à gauche :
        # (0:id 1:url 2:fichier icône 3:texte alt 4:balloon help) 
        liOp =(
        ("ret","/index","bouton10","home",_("Accueil")),
        ("car","/notDec/prospection","bouton1","carotteuse",_("Prospection (Carottage)")),
        ("mic","/notDec/microscope","bouton2","microscope",_("Microscope")),
        ("lab","/notDec/laboratoire","bouton3","labo",_("Laboratoire")),
        ("cle","%s/frame.htm" % Glob.keyPath,"bouton5","clef",_("Clé de détermination")),
        ("gra","/notDec/diagrammes","bouton6","graphes",_("Diagrammes polliniques")),
        ("dat","/start/datationC14","bouton7","c14",_("Datation au carbone 14")),
        ("cli","/start/interpretPaysa","bouton11","thermom",_("Reconstitution des climats")),
        ("doc",Glob.docPath,"bouton9","bouquin",_("Documentation")),
        ("que","/start/questionnaire","bouton8","question",_("Questionnaire")))
            
        tools = '<DIV ID="tools">'
        styles = '<style type="text/css">'
        y = 75         # positionnement vertical des icônes
        for i in liOp:
            tools +='\n<A HREF="%s" TITLE="%s">'\
            '<IMG ID="%ssIcon" SRC="/static/%s.jpg" ALT="%s" '\
            '''onmouseover="showLeftPopup(event, '%s')">'''\
            '<IMG ID="%shIcon" SRC="/static/%s_b.jpg"'\
            '''onmouseout="hideLeftPopup('%s')"> </A>''' %\
            (i[1],i[4],i[0],i[2],i[3],i[0],i[0],i[2],i[0])
            styles +='\n#%ssIcon  { position:absolute; top:%s; left:3px; }'\
            '\n#%shIcon  { position: absolute; display: none;'\
            ' top: %s; left: 3; z-index: 5; }' % (i[0], y, i[0], y)
            y += 55             # position verticale de l'icône
        tools +='<br>'\
        '<a href="http://www.mozilla.com/en-US/" title='\
        '"Preferred browser"><img id="ff" src="/static/firefo2.gif"></a></DIV>'
        styles +='\n#ff { position:absolute; top:640; left:3px; }</style>\n'

        # Construction de la page HTML :
        #'<META http-equiv=Content-Type content="text/html; charset=iso-8859-1">\n'\
        return ttt("com","comHeader") % (styles, tools)

    def footer(self):
        return '''\n</DIV></BODY></HTML>'''

    def _before(self, methodName, method):
        # Cette méthode est invoquée chaque fois qu'une autre méthode
        # quelconque de la classe est appelée, et elle s'exécute en premier.
        # C'est elle qui insère au début de chaque page le code défini
        # dans la méthode <header> ci-dessus. Il faut cependant inhiber
        # ce mécanisme dans le cas des méthodes <header> et <footer>
        # elles-mêmes, sinon on obtient une boucle de récursion infinie !

        if methodName in ('header', 'footer', '__init__'):
            return CONTINUE, ''
        else:
            return CONTINUE, self.header()

    def _after(self, methodName, method):
        # Cette méthode est invoquée après chacune des autres. Elle permet
        # donc d'insérer un code commun en bas de chaque page.
        # Mêmes restrictions que pour la méthode <_before> ci-dessus.

        if methodName in ('header', 'footer', '__init__'):
            return CONTINUE, ''
        else:
            return CONTINUE, self.footer()

    def default(self, *args, **kwArgs):
        # Page affichée par défaut si la page demandée est introuvable.
        # args = tuple contenant les arguments transmis sans étiquettes.
        # kwArgs = dico contenant les arguments avec étiquettes.

        at, ant = "", ""
        if len(args):
            at ="<p>Référence(s) de la page : "
            for a in args:
                at += "%s " % (a)
            at +="</p>"
        if len(kwArgs):
            ant ="<p>Arguments transmis : "
            for k, a in kwArgs.items():
                ant += "%s=%s &nbsp;&nbsp;" % (k, a) 
            ant +="</p>"
        return '</BR><H3>Erreur : page inexistante ou non "exposée".</H3>'\
        '%s%s' % (at, ant)

    default.exposed =True

class PageUpdate(Page):
    # Les pages produites à l'aide de cette classe ne comporteront pas
    # l'en-tête et le pied de page communs.

    def _before(self, methodName, method):
        return CONTINUE, ''
    
    def _after(self, methodName, method):
        return CONTINUE, ''


class PageAccueil:    
    # rem : classe non dérivée de <Page> (sans menus ni outils).

    def __init__(self):
        # instanciation des principaux objets gestionnaires de pages :
        self.start =StartPage(self)
        self.exploi =Exploitation(self)
        self.micro =Microscope(self)
        self.notDec =NotDecorated(self)

    def index(self):
        # Cherrypy invoquera cette méthode comme URL racine (/)
        # C'est donc ici que la session commence
        # Quel browser le client utilise-t-il ?
        # print cherrypy.request.headers     # dico avec infos de la requête
        if "MSIE" in cherrypy.request.headers['User-Agent']:
            cherrypy.session["oldBrowser"] =1
            nextP ="/ieWarning"
        else:
            cherrypy.session["oldBrowser"] =0
            nextP ="/start/userPrompt"
        cherrypy.session["nom"] =None
        cherrypy.session["profond"] =1000000
        return ttt("com","index") % (Glob.indexFile, nextP)

    index.exposed =True

    def ieWarning(self):
        return '<HTML><HEAD><META http-equiv=Content-Type content="text/html; '\
        'charset=utf-8">\n</HEAD><BODY>\n%s\n</BODY></HTML>' %\
        ttt("msg", "ieWarning")

    ieWarning.exposed =True
    
    def palynoTeam(self):
        return ttt("com", "palynoTeam") % \
        (_("Conception scientifique et réalisation :"), ttt("msg", "licence"))
        
    palynoTeam.exposed =True
    
    def pngOK(self):
        cherrypy.session["oldBrowser"] =0
        return self.start.userPrompt()
        
    pngOK.exposed =True

    def default(self, *args, **kwArgs):
        # Page affichée par défaut si la page demandée est introuvable.
        return _("</BR><H3>Erreur : la page demandée n'existe pas.</H3>")

    default.exposed =True

    def choixBordure(self):
        #self.micro.changeBordure()
        self.micro.bordureNoire = not self.micro.bordureNoire
        return self.micro.montrerChamp()

    choixBordure.exposed =True
    
    def dimChamp(self):
        self.micro.bigSize = not self.micro.bigSize
        return self.micro.montrerChamp()

    dimChamp.exposed =True


class NotDecorated(PageUpdate):
    # classe permettant de définir des pages 'relais' (sans header/footer)
    def __init__(self, boss):
        self.boss =boss             # réf. PageAccueil
        self.start =boss.start      # réf. StartPage
        self.exploi =boss.exploi    # réf. page exploitation
        self.micro =boss.micro      # réf. Microscope

    def diagrammes(self):
        # L'utilisateur a cliqué sur l'outil <diagrammes> :
        p = cherrypy.session.get("profond", 1000000)
        if p >999999:      # Aucune profondeur n'a encore été choisie
            # L'utilisateur s'est-il identifié ?
            rUsr = cherrypy.session.get("ref_usr", None)
            if rUsr:
                return self.exploi.roleOutilDiagram()
            else:
                return self.start.userPrompt()

        return self.exploi.showPolDiagr()
    
    diagrammes.exposed =True

    def laboratoire(self):
        # L'utilisateur a cliqué sur l'outil laboratoire
        p = cherrypy.session.get("profond", 1000000)
        if p >999999:      # Aucune profondeur n'a encore été choisie
            # L'utilisateur s'est-il identifié ?
            nom = cherrypy.session.get('nom', None)
            if nom:
                return self.exploi.roleOutilLabo()
            else:
                return self.start.userPrompt()

        # Si animation déjà visionnée, on montre la prépa. :     
        return self.exploi.showPreparation()
            
    laboratoire.exposed =True

    def prospection(self, rAns =0, answer=None, variable=""):
        # L'utilisateur a cliqué sur la carotteuse, ou entré une réponse
        # après avoir choisi le biotope "tourbière".
        region = cherrypy.session.get("region", None)
        okTourb =cherrypy.session.get("okTourb", 0)
        rUsr =cherrypy.session.get("ref_usr", None)
        rAns =int(rAns)
        rQue =1
        if not rUsr:
            return self.start.userPrompt()
        if not region:
            return self.start.greetUser()
        if not okTourb:
            return self.start.biotope()
        if answer is not None:
            # Mémorisation de la réponse entrée :
            res =memoriseReponse(rAns, answer, variable, rQue)
            if res: return res               # error msg
            okTourb =(2, 3)[len(answer) >5]
            cherrypy.session["okTourb"] =okTourb
        if okTourb >2: return self.start.carottes()  # => on continue
        else: return self.start.tourbiere()          # => on repose la question
    
    prospection.exposed =True
    
    def questionLabo(self, rAns =0, answer=None, variable=""):
        # L'utilisateur a entré une réponse à la question posée dans 'Labo'
        #okQueLabo =cherrypy.session.get("okQueLabo", 0)
        rAns =int(rAns)      # réf. réponse antérieure éventuelle 
        rQue =2              # réf. question
        if answer is not None:
            # Mémorisation de la réponse entrée :
            res =memoriseReponse(rAns, answer, variable, rQue)
            if res: return res               # error msg
            okQueLabo = (2,3)[len(answer) >5]
            #cherrypy.session["okQueLabo"] =okQueLabo
        if okQueLabo >2: return self.microscope()
        else: return self.exploi.showPreparation()

    questionLabo.exposed =True

    def reponseChoisie(self, rQue =0, rAns =0, answer="", variable =""):
        # L'utilisateur a entré une réponse à une question choisie par lui
        rQue =int(rQue)              # réf. question
        rAns =int(rAns)              # réf. réponse antérieure
        if len(answer) >4:
            # Mémorisation de la réponse entrée :
            res = memoriseReponse(rAns, answer, variable, rQue)            
            if res: return res               # error msg
        return self.start.questionnaire()
        
    reponseChoisie.exposed =True

    def microscope(self):
        # L'utilisateur a cliqué sur l'outil microscope
        
        p = cherrypy.session.get("profond", 1000000)
        if p >999999:      # Aucune profondeur n'a encore été choisie
            # L'utilisateur s'est-il identifié ?
            nom = cherrypy.session.get('nom', None)
            if nom:
                return ttt("msg","outilMicro")
                # return self.start.choisissezEchantillon()
            else:
                return self.start.userPrompt()

        # Si un échantillon est en cours d'observation, on continue avec lui :
        if cherrypy.session.get("memPol", None):
            return self.micro.montrerChamp()
        
        # Préparation de l'échantillon de pollens dans son ensemble.
        # Il s'agira d'une liste de tuples contenant chacun :
        # 0:taxon  1:taille de l'image en pixels  2:image  3:index image
        polPres =pollensPresents(Glob.fieldSize1)
        cherrypy.session["polPres"] =polPres
        # Établir aussi une liste des taxons présents :
        taxPres =[]
        for t in polPres:
            tp = t[0]
            if tp not in taxPres:
                taxPres.append(tp)
        cherrypy.session["taxPres"] =taxPres
       
        # Préparation des champs microscopiques:
        chMicro =[]
        while 1:
            # nouvelle disposition de grains de pollen dans le champ :
            disPol =dispositionPollens()
            if len(disPol):
                chMicro.append(disPol)
            else:
                break
            
        ## Debug info : pollens présents dans chaque champ :
        #diTT = {}
        #for n, d in enumerate(chMicro):
            #diT = {}
            #print "champ %s : " % (n),
            #for p in d:
                #t = p[3][0]
                #diT[t] = diT.get(t,0) +1
                #diTT[t] = diTT.get(t,0) +1                
            #for k,v in diT.items():
                #print "%s %s  " % (v, k),
            #print
        #print "total : ",
        #for k,v in diTT.items():
            #print "%s %s  " % (v, k),
        #print
        
        # ----------------------------------------               
        # Conservation de la liste des champs micro. construits :        
        cherrypy.session["memPol"] =chMicro
        # Premier champ d'observation :
        cherrypy.session["champ"] =0
        # Préparation d'une liste de listes, qui mémorisera les grains
        # déterminés dans chaque champ :
        cherrypy.session["detPol"] =[[] for i in range(len(chMicro))]
        # Aucun exemplaire de grain de pollen n'est cliqué au départ :
        cherrypy.session["exempl"] =None
        # Initialisation du nombre total de grains déjà déterminés :
        cherrypy.session["totalDet"] =[]

        return self.micro.montrerChamp()
        
    microscope.exposed =True

    def chooseUser(self, user =0, lp =0, newUser =0):
        if newUser:
            # encodage d'un nouvel utilisateur :
            return self.start.inputUser()
        elif user:
            # user dans la liste - récupération de ses données :
            qy ="SELECT nom, prenom, passwd FROM users WHERE r_usr =%s" % (user)
            res = resRequeteBD(qy) 
            if type(res) == stringType: return res       # erreur SQL
            nom, prenom, passwd = res[0][0], res[0][1], res[0][2]            
            if not lp or passwd != lp:
                return '<H4>%s</BR></BR>'\
                '<A HREF="/start/userPrompt">%s</a></H4>' %\
                (_("Mot de passe non fourni ou erroné."),
                 _("Retour à la page d'accueil"))
            cherrypy.session["ref_usr"] =user
            return self.start.greetUser(nom, prenom)
        else:
            # message d'erreur car aucun choix n'a été fait :
            return self.start.greetUser()
 
    chooseUser.exposed =True    

    def newUser(self, nom =None, prenom =None, pwd1=None, pwd2 =None):
        if not nom or not prenom:
            # N'a pas fourni son nom ou son prénom :
            return '<H4> Veuillez SVP choisir un nom et un prénom.'\
            '/BR></BR> <A HREF="/start/userPrompt">'\
            "Retour à la page d'accueil </a></H4>"
        if pwd1 != pwd2 or not pwd1:
            return '<H4> Mot de passe non fourni ou mal confirmé.'\
            '</BR></BR> <A HREF="/start/userPrompt">'\
            "Retour à la page d'accueil </a></H4>"
        # Mémorisation dans la liste des utilisateurs :
        rUsr =Glob.DB.nextNum("users", "r_usr")
        qy ="INSERT INTO users (r_usr, nom, prenom, passwd) "\
        "VALUES (%s, '%s', '%s', '%s')" % (rUsr, nom, prenom, pwd1)
        res =exeRequeteBD(qy)
        if res: return res          # error msg
        # Accès concédé à la suite du programme :
        cherrypy.session["ref_usr"] =rUsr
        return self.start.greetUser(nom, prenom)

    newUser.exposed =True

    def choixRegion(self):
        # L'utilisateur s'est-il identifié ?
        nom = cherrypy.session.get('nom', None)
        if nom:
            # Le choix de la région est désormais reporté plus loin :
            #return self.start.mainMap()
            # Désormais, on commence à prospecter directement en Belgique : 
            cherrypy.session["region"] = "be"
            return self.start.region()
        else:
            return self.start.userPrompt()

    choixRegion.exposed =True

    def belgium(self):
        cherrypy.session["region"] = "be"
        return self.start.region()

    belgium.exposed =True
    
    def france(self):
        cherrypy.session["region"] = "fr"
        return self.start.region()

    france.exposed =True

    def poland(self):
        cherrypy.session["region"] = "po"
        return self.start.region()

    poland.exposed =True
    
    def turkey(self):
        cherrypy.session["region"] = "tr"
        return self.start.region()

    turkey.exposed =True
    
    def greece(self):
        cherrypy.session["region"] = "gr"
        return self.start.region()

    greece.exposed =True

    def romania(self):
        cherrypy.session["region"] = "ro"
        return self.start.region()

    romania.exposed =True

    def switzerland(self):
        cherrypy.session["region"] = "ch"
        return self.start.region()

    switzerland.exposed =True

class NoDecorations(PageUpdate):
    # classe permettant de définir des pages 'relais' (sans header/footer)
    def __init__(self, boss):
        self.micro =boss
        self.start =boss.start              # réf. StartPage
        self.exploi =boss.exploi            # réf. Pages Exploitation

    def countCompleted(self):
        # L'utilisateur a cliqué sur le bouton de sortie de la fenêtre
        # correspondant aux observations microscopiques :
        okLabo = cherrypy.session.get("okLabo", None)
        if not okLabo:
            return self.start.carottes()
        # Il faut avoir déterminé au moins la moitié des taxons
        # présents pour pouvoir sortir :
        totalDet =cherrypy.session["totalDet"]     # taxons déjà déterminés
        taxPres =cherrypy.session["taxPres"]       # taxons présents
        if len(totalDet) > len(taxPres)/2:   
            return self.micro.detCompleted()
        else:
            return self.micro.montrerChamp()
        
    countCompleted.exposed =True

    def retourClePollens(self, choosed="", submit=""):
        # Point de retour de la clé de détermination. Le paramètre choosed
        # contiendra le taxon choisi, ou rien si annulation
        return self.choosedPollen(taxon=choosed)

    retourClePollens.exposed =True

    def choosedPollen(self, taxon="", index=0, clickx=0, clicky=0):
        if clickx:
            # Un grain de pollen a été pointé. Mémorisation :
            cherrypy.session["exempl"] =taxon
            # <taxon> contient à ce stade l'identité individuelle d'un grain
            pur_taxon =taxon.split("_")[0]      # extraction du taxon 'pur'
            cherrypy.session["chPollen"] =pur_taxon
            cherrypy.session["indexPol"] =int(index)
            # Ajout d'un cercle de sélection dans l'image du champ microscopique :
            cherrypy.session["chPol_XY"] =(int(clickx), int(clicky))
            return self.micro.montrerChamp(taxon)
        else:
            # Un taxon a été choisi dans la planche de référence, ou
            # dans la clé de détermination.
            # <taxon> contient bien dans ce cas le taxon 'pur' 
            # Comparaison avec le pollen pointé dans le champ micro : 
            chPollen =cherrypy.session.get("chPollen", "-")
            detErr =cherrypy.session.get("detErr",0)
            
            if chPollen == taxon:
                # Mémoriser ce grain comme étant déterminé correctement :
                index =cherrypy.session["indexPol"]
                champ =cherrypy.session["champ"]                
                if not index in cherrypy.session["detPol"][champ]:
                    # Ce grain n'avait pas encore été déterminé. On mémorise : 
                    cherrypy.session["detPol"][champ].append(index)
                    # On complète la liste des taxons déjà déterminés :
                    totalDet =cherrypy.session["totalDet"]
                    if not taxon in totalDet:
                        totalDet.append(taxon)
                    # Si l'on a déterminé les 4/5 des taxons présents,
                    # on sort automatiquement de la page : 
                    taxPres =cherrypy.session["taxPres"]
                    if len(totalDet) > len(taxPres)*.8:   
                        return self.micro.detCompleted()
                        
                cherrypy.session["chPollen"] ="-"
                return self.micro.montrerChamp()
            else:
                detErr +=1
                cherrypy.session["detErr"] =detErr
                return self.micro.badDetermination(taxon, chPollen)
  
    choosedPollen.exposed =True

    def nextField(self):
        memPol =cherrypy.session["memPol"]  # liste des champs micro
        if cherrypy.session["champ"] < len(memPol)-1:
            cherrypy.session["champ"] += 1
        cherrypy.session["chPollen"] ="-"
        cherrypy.session["chPol_XY"] =None

        return self.micro.montrerChamp()

    nextField.exposed =True

    def previousField(self):
        if cherrypy.session["champ"] >0:       # champ courant
            cherrypy.session["champ"] -= 1
        cherrypy.session["chPollen"] ="-"
        cherrypy.session["chPol_XY"] =None

        return self.micro.montrerChamp()

    previousField.exposed =True
        
class Microscope(Page):
    
    def __init__(self, boss):
        
        self.boss =boss                     # réf. PageAccueil
        self.start =boss.start              # réf. StartPage
        self.exploi =boss.exploi            # réf. Pages Exploitation        
        self.noDeco =NoDecorations(self)    # page "relais"
   
    #def retourClePollens(self, choosed):
        #correct =cherrypy.session["chPollen"]
        #return "Taxon choisi dans la clé : %s<br>"\
               #"Taxon correct : %s" % (choosed, correct)

    #retourClePollens.exposed =True


    def badDetermination(self, exFourni, correct):
        # print "taxon fourni : %s correct : %s" % (exFourni, correct)
        fourni, correct = exFourni[:3], correct[:3]
        # indice correspondant au message de correction à afficher :
        ind = Glob.indComCor.get((fourni, correct), 98)
        # message de correction :
        comment =Glob.diComCor[ind][0]
        return "<h3><br />" +\
        _("Votre détermination est erronée.") +"<br /><br />" +\
        comment +"<br /><br />" +\
        _("Veuillez s.v.p. utiliser la clé de détermination pour éviter de vous tromper.") +\
        '<br /><br /><a href="/micro/montrerChamp">' +\
        _("Cliquez ici pour continuer") +"</a></h3>"

    badDetermination.exposed =True
    
    def detCompleted(self):
        # On a terminé les déterminations d'un niveau
        depth =cherrypy.session["profond"]
        nbErr =cherrypy.session.get("detErr", 0)
        # Mémorisation :
        rUsr =cherrypy.session["ref_usr"]
        qy ="SELECT r_pro, errors FROM depths WHERE r_usr =%s AND depth =%s" %\
        (rUsr, depth)
        res = resRequeteBD(qy) 
        if isinstance(res, str): return res       # erreur SQL
        if len(res):
            rPro =res[0][0]
            nbErr += res[0][1]
            qy ="UPDATE depths SET checked=1, errors =%s WHERE r_pro =%s" %\
            (nbErr, rPro)
        else:
            n = enregSuivant("depths", "r_pro")
            qy ="INSERT INTO depths (r_pro, r_usr, depth, checked, errors, "\
            "dated, biome) VALUES (%s, %s, %s, %s, %s, %s, '%s')" %\
            (n, rUsr, depth, 1, nbErr, 0, "")
        res =exeRequeteBD(qy)
        if res: return res          # error msg
        cherrypy.session["detErr"] =0
        
        # Tableau avec les nombres de pollens présents
        tot =0
        proportions ="<table class=left cellspacing=0>"
        for t in Glob.taxons:
            txn =t[0]                    # taxon
            tna =txn[:3]                 # alias (taxon abrégé)
            if tna =="cra": continue     # ignorer les "crasses"
            # nombre de grains comptés = proportion présente (%) / 2 : 
            p =Glob.diProfTax.get((depth, tna), 0) /2
            txn =txn.capitalize()
            if tna =="che": txn ="Chenopodiaceae"
            coulF ={"a":"coulArbres", "h":"coulHerbes", "z":"white"}[t[3]]
            proportions += "\n<tr class=%s><td>%s</td><td>%s</td><tr>" %\
                        (coulF, txn, p)
            tot += p       # total des pollens comptés
        proportions +='\n</table>'\
        '<img class=leftMenu src="/static/user%s/histogr_polli0.png">' % rUsr
        # Construction de l'histogramme correspondant dans l'image PNG :
        histogrammePollens(depth)

        if nbErr: errMsg =_("(vous vous êtes cependant trompé %s fois)") % (nbErr)
        else: errMsg =""
        return ttt("msg","detCompleted") % (depth +5, errMsg, tot, proportions)
        
    detCompleted.exposed =True
    
    def montrerChamp(self, exemplaire =None):
        # Si un nom d'exemplaire est fourni,
        # le champ micro. secondaire contiendra qqc. Sinon, rien.
        if not exemplaire:
            exemplaire = cherrypy.session.get("exempl", None)
        
        # Reconstruction du champ micro.
        # La fonction ci-dessous génère un fichier image PNG et 
        # renvoie une chaîne contenant l'"image map" correspondante.
        # Les champs de cette "map" invoquent tous la fonction JavaScript
        # selectedPollen() en lui transmettant le nom de l'exemplaire
        # de grain pointé :
        microMap =champMicroGlobal()
    
        # La fonction ci-après (re)construit le champ
        # micro secondaire (n coupes optiques dans le même grain agrandi).
        # Les images sont placées dans une série de fichiers.
        # La fonction renvoie le code javascript qui permettra au navigateur 
        # client de précharger ces images et les faire défiler.
        # Si <exemplaire> est None, la fonction produit une image vide : 
        aniScript = champMicroDetail(Glob.fieldSize2, exemplaire)

        profondeur =cherrypy.session["profond"]
        ch1W =Glob.fieldSize1       # taille du champ micro général
        ch2W =Glob.fieldSize2       # taille du champ secondaire
        ch2L =5 +(ch1W-ch2W)/2      # pos. horiz. du champ secondaire
        refH = ch1W+ch2W            # hauteur de la planche de référence
        refW = refH/1.38            # largeur        "  "
        refLeft = ch1W+10           # marge gauche   "  "
        b1Left =ch1W/2-120          # pos. horizontales des boutons de champ
        b2Left =ch1W/2+100
        b1Top =ch1W-8               # pos. verticales "  "
        b2Top =ch1W-8
        b3Top = ch1W+ch2W/2-80      # pos. verticales des boutons de focus
        b4Top = ch1W+ch2W/2+110
        b5Left = ch1W-32            # pos. horizontale bouton exit
        legTop = ch1W+40+ch2W       # position verticale de la légende

        # Chemin de l'image du champ micro global générée :
        imFile ="/static/pollens/champ_micro.png"
        # Chemin des images du champ agrandi :
        detFile ="/static/pollens/temp/champ_vide_1.png"
        # Boutons :
        b1File ="/static/back.png"
        b2File ="/static/forward.png"
        b3File ="/static/up.png"
        b4File ="/static/down.png"
        b5File ="/static/up.png"

        return '''
<style type="text/css">
#chMicro { position:absolute; top:25; left:5; }
#exitB   { position:absolute; top:25; left:%s; }
#prevC   { position:absolute; top:%s; left:%s; }
#nextC   { position:absolute; top:%s; left:%s; }
#refPoll { position:absolute; top:25; left:%s; }
#chSec   { position:absolute; top:%s; left:%s; }
#prevB   { position:absolute; top:%s; left:%s; }
#nextB   { position:absolute; top:%s; left:%s; }
#legende { position:absolute; top:%s; left:5; width:910;}
</style>

<form name="formulr" action="/micro/noDeco/choosedPollen" method=post>
%s
<img id="chMicro" src="%s" width="%s" height="%s"
ALT="Microscopic field" usemap="#chmap" />
<map name="chmap">
%s
</map>

<input type="hidden" name="taxon" />
<input type="hidden" name="index" />
<input type="hidden" name="clickx" />
<input type="hidden" name="clicky" />
<a href="/micro/noDeco/countCompleted" title="%s">
<img id="exitB" src="%s" ALT="button" /></a>
<a href="/micro/noDeco/previousField">
<img id="prevC" src="%s" ALT="button" /></a>
<a href="/micro/noDeco/nextField">
<img id="nextC" src="%s" ALT="button" /></a>        
%s
%s
<img id="chSec" src="%s" width="%s" height="%s"
ALT="expanded view of the selected pollen" />
<a href ="javascript:previousImage()"><img id="prevB" src="%s" ALT="button" /></a>
<a href ="javascript:nextImage()"><img id="nextB" src="%s" ALT="button" /></a>
<div id="legende">%s<br>
<a href="%s"><h3>%s</h3><p>%s</p></a></div>
</form>
<SCRIPT>
%s
function selectedPollen(event, txn, ind)
{   var x =event.clientX +document.body.scrollLeft
    var y =event.clientY +document.body.scrollTop
    document.formulr.taxon.value =txn
    document.formulr.index.value =ind
    document.formulr.clickx.value =x
    document.formulr.clicky.value =y
    document.formulr.submit()
    event.cancelBubble =true
}
function selectedTaxon(id)
{   document.formulr.taxon.value =id
    document.formulr.clickx.value =''
    document.formulr.clicky.value =''
    document.formulr.submit()
}
function showBigPopup(event, id)
{   var x =event.clientX +document.body.scrollLeft;
    var y =event.clientY +document.body.scrollTop;
    var popDiv =document.getElementById(id + "BigPop");
    if (y >window.innerHeight +document.body.scrollTop -210)
        y =window.innerHeight +document.body.scrollTop -210;
    if (x >window.innerWidth +document.body.scrollLeft -370)
        x = x-390;
    popDiv.style.left =x-30;  popDiv.style.top =y-30;
    popDiv.style.display ="block";
    event.cancelBubble =true;
}
function hidePopup(id)
{   var pop =document.getElementById(id + "BigPop") ;
    if (pop)
        pop.style.display ="none";
}
function nothing()
{}
</SCRIPT>''' %\
       (b5Left, b1Top, b1Left, b2Top, b2Left, refLeft, ch1W+30, ch2L,
        b3Top, ch2L, b4Top, ch2L, legTop,
        ttt("msg", "obsPresenta") % (profondeur +5),
        imFile, ch1W, ch1W, microMap, _("Terminer"), b5File, b1File, b2File,
        ttt("com", "plRefMap"), Glob.taxPop,
        detFile,  ch2W, ch2W, b3File, b4File,
        ttt("msg","obsInstruct"),
        Glob.docPath, _("Consulter la documentation :"),
        ttt("msg","docMicro"), aniScript)

    montrerChamp.exposed =True
    
class Exploitation(Page):
    def __init__(self, boss):
        self.boss =boss             # réf. PageAccueil
        self.start =boss.start
        
    def showPreparation(self):
        cherrypy.session["okLabo"] =True     # => la prépa. existe
        p = cherrypy.session["profond"] +5
        # On va poser une question de rélexion, puis continuer.
        # Aurait-il déjà répondu auparavant ? :
        rQ =2         # réf. question
        # Rechercher si une réponse avait déjà été fournie :
        rep = trouveReponse(rQ)        # liste de 1 tuple dans ce cas
        answer, rAns, var =rep[0]

        return ttt("com", "showPreparation2") %\
        (_("Profondeur"),
         " : -%s cm." % (p),
         ttt("msg","showPreparation"),
         ttt("quest", "2"),
         answer, _("Mémoriser la réponse"), rAns,
         ttt("msg", "directivesQuest"),
         Glob.docPath, _("Consulter la documentation :"),
         _("Comment extraire les spores et pollens des sédiments tourbeux ou lacustres ?"))   

    showPreparation.exposed =True
        
    def showPolDiagr(self):
        # Annuler les reconstitutions précédentes :
        cherrypy.session["pays_started"] =False
        #cherrypy.session["pays_taxlist"] =[]

        # Retrouver la liste des profondeurs déjà prospectées :
        rUsr =cherrypy.session["ref_usr"]
        profProsp =profondeursProspectees()
        # Construire le diagramme pollinique correspondant
        # la fonction ci-après renvoie l'"image map" pour les boutons):    
        map = diagrammePollinique(profProsp)
        return '%s%s<p>%s</p><br><a href="%s"><h3>%s</h3><p>%s</p></a>' %\
               (ttt("msg", "showPolDiagr"),
                ttt("com", "showPolDiagr2") % (rUsr, map),
                ttt("msg","datation_4"),
                Glob.docPath, _("Consulter la documentation :"),
                _("Comment construit-on un diagramme pollinique ?"))
    
    showPolDiagr.exposed =True
    
    def showPaysage(self, profond =1000000, taxon="", numQ=[], varQ=[],
                    answ=[], rAQ=[]):
        # Reconstitution du paysage ancien correspondant à la profondeur donnée
        # Cette méthode est rappelée chaque fois que l'utilisateur clique sur
        # un symbole de taxon => insertion de nouveaux éléments dans le paysage
        # <answ> peut contenir une liste des réponses aux questions posées
        # <numQ> peut contenir une liste des numéros des questions posées
        # <rAQ> peut contenir une liste des réf. réponses déjà fournies
        # <varQ> peut contenir une liste de variables associées aux questions

        profond =int(profond)
        if isinstance(numQ, str): numQ =[numQ]
        if isinstance(varQ, str): varQ =[varQ]
        if isinstance(answ, str): answ =[answ]
        if isinstance(rAQ, str): rAQ =[rAQ]

        #print "numQ", numQ, type(numQ), "varQ", varQ, type(varQ), "answ", answ, "rAQ", rAQ, type(rAQ)        
        #print "numQ", numQ, "varQ", varQ, "answ", answ, "rAQ", rAQ       
        
        if len(numQ):
            for i, ans in enumerate(answ):
                # réponse antérieure, n° de question, variable associée :
                rA, num, var = int(rAQ[i]), int(numQ[i]), varQ[i]
                # Mémorisation de la nouvelle réponse entrée éventuelle :
                res =memoriseReponse(rA, ans, var, num)
                
        # Présentation éventuelle de question(s) en bas de page :
        page2 =""
        for qtp in Glob.listeQuesPays:         # parcourir les questions
            liPro =qtp[0]           # liste de profondeurs concernées par la q.
            # Passer les questions qui ne sont pas liées à cette profondeur
            #if profond not in liPro: continue
            if liPro[0] >=0 and profond not in liPro: continue
            taxQ =qtp[1]            # taxon concerné éventuel
            # Passer les questions concernant un autre taxon que celui cliqué
            if taxQ !="-" and taxon[:3] != taxQ: continue
            numQ =qtp[2]            # N° de la question
            p5 =profond +5
            qq =trouveQuestion(numQ)  # liste de tuples (q, answ, rAQ, var)
            for tu in qq:
                quest, answ, rAQ, variable = tu
                if "%s" in quest:            # question avec variable
                    quest =quest % p5
                    if variable ==str(p5):   # profondeur déjà rencontrée                      
                        break
                    else:
                        # réponse déjà fournie, mais pour autre profondeur
                        rAQ, answ = 0, ""    
                else: p5 =""
            page2 +='<p class=left>%s <br>'\
            '<INPUT TYPE=hidden NAME="rAQ" VALUE="%s">'\
            '<INPUT TYPE=hidden NAME="numQ" VALUE="%s">'\
            '<INPUT TYPE=hidden NAME="varQ" VALUE="%s">\n'\
            '<TEXTAREA NAME="answ" cols=60 rows=2>%s</TEXTAREA>'\
            '<br>(%s)</p>' % (quest, rAQ, numQ, p5, answ,
            _("Cliquez sur un taxon quelconque pour enregistrer la réponse"))
        
        # Préparation des images :
        started =cherrypy.session.get("pays_started", False)
        if not started:
            histogrammePollens(profond, 1)
            cherrypy.session["pays_started"] =True
            cherrypy.session["memPA"] =[]
            cherrypy.session["memPH"] =[]
            cherrypy.session["memIA"] ={}
            cherrypy.session["memIB"] ={}
            cherrypy.session["memIC"] ={}
        # Régénération du paysage, avec ajout du taxon <taxon> :
        paysageVirtuel(profond, taxon= taxon)
        # Préparation du reste de la page :
        lTax =Glob.diLanDat[-1]         # taxons dans l'ordre
        lanData = Glob.diLanDat[profond]
        lAH  =Glob.diLanDat[-3]         # indications 'arbre' ou 'herbacée'
        
        # Répertoire spécifique de l'utilisateur courant :
        workDir ="/static/user%s" % (cherrypy.session["ref_usr"])
        page = ttt("msg","showPaysage") % (profond +5) +\
               ttt("com","showPaysage2") % (profond, workDir, workDir)
        # Symboles des taxons de plantes herbacées :
        for i, taxon in enumerate(lTax):
            if lAH[i] != "h": continue         
            if taxon !="poaceae" and lanData[i] ==0: continue
            # *** On ne souhaite plus des symboles # pour les # herbacées : ****
            #page +='''\n<a class=noUnderL href="javascript:selectedTaxon('%s')">'''\
            #'\n<div class="taxonH"><img src="/static/m_%s.gif"> '\
            #'<img src="/static/m_%s_2.gif"><br>%s</div></a>' %\
            #(taxon, taxon, taxon, taxon.capitalize())
            page +='\n<a class=noUnderL '\
            '''href="javascript:selectedTaxon('%s')">\n'''\
            '<div class="taxonH"><img src="/static/m_%s.gif"><br>%s</div></a>'\
            % (taxon, taxon, taxon.capitalize())
        page +='\n</div><br>\n<div class=left>'
        # Symboles des taxons d'arbres :
        for i, taxon in enumerate(lTax):
            if lAH[i] != "a": continue
            # ignorer les taxons absents à cette profondeur, sauf Pinus (10)
            # qui fait l'objet d'une question aux profondeurs -95 à -135 :
            if i ==10 and profond >85 and profond < 135: pass
            elif lanData[i] ==0: continue
            page +='\n<a class=noUnderL '\
            '''href="javascript:selectedTaxon('%s')">'''\
            '<div class="taxonA"><img src="/static/m_%s.gif"><br>'\
            '%s</div></a>' % (taxon, taxon, taxon.capitalize()) 
        # Lien vers la doc :
        page +='<div class=left><a href="%s"><h3>%s</h3><p>%s</p></a></div>' %\
        (Glob.docPath, _("Consulter la documentation :"), ttt("msg","taxonsHF"))

        return '%s\n</div>%s</form>' % (page, page2)
    
    showPaysage.exposed =True

    def roleOutilDiagram(self):
        return ttt("msg", "roleOutilDiagram")

    roleOutilDiagram.exposed =True
    
    def roleOutilLabo(self):
        return ttt("msg", "roleOutilLabo")

    roleOutilLabo.exposed =True

    def questionChoisie(self, choosed=""):
        # L'utilisateur a choisi une question du questionnaire général :
        nq, vc, quest = tuple(choosed.split("#"))     # n° q., var, question
        # Rechercher si une réponse avait déjà été fournie auparavant :
        rep = trouveReponse(nq)        # liste de tuples (1 pour chaque valeur
                                       # de la variable associée éventuelle) 
        for answer, rAns, var in rep:
            if var !="§":              # question avec variable
                if var ==vc: break     # choisir la bonne !
        return ttt("com", "questionChoisie2") %\
        (nq, quest, _("Veuillez entrer ou corriger votre réponse :"),
         answer, _("Mémoriser la réponse"), nq, rAns)   
    
    questionChoisie.exposed =True

class StartPage(Page):

    def __init__(self, boss):
        self.boss =boss             # réf. PageAccueil

        # dico des photos et descriptifs des régions :
        self.liPop =dict(
            be =("climatic6",
                 _("Belgique : plateau des Hautes-Fagnes"),
                 ('localisationHF','vue_generale','Boulaie','hetraie','pessieres','TH')),
            po =("climatic7",
                 _("Pologne : réserve de *******"),("",)),
            fr =("climatic7",
                 _("France : tourbière de *******"),("",)),
            ro =("climatic7",
                 _("Roumanie : réserve de *******"),("",)),
            it =("climatic7",
                 _("Italie : tourbière de *********"),("",)),
            tr =("climatic7",
                 _("Turquie : réserve de ********"),("",)),
            gr =("climatic7",
                 _("Grèce : réserve de ******"),("",)),
            ch =("climatic7",
                 _("Suisse : réserve de ********"),("",)))

    def userPrompt(self):
        # Demande d'identification de l'utilisateur connecté
        # On commence par vérifier que le serveur SQL est OK :
        msg = "<P> %s </P>"  % Glob.DB.dbi_msg    # message: OK/Oups
        if not Glob.DB.connexion:
            if "Données inaccessibles" in msg:     # connexion ratée
                return '%s </BR></BR>Si le problème persiste,</BR>'\
                'Veuillez contacter <BIG>http://inforef.be</BIG>' % msg
        # remplissage de la boîte de liste avec les noms d'utilisateurs :
        qy ="SELECT r_usr, nom, prenom FROM users ORDER BY nom"        
        resReq = resRequeteBD(qy) 
        if type(resReq) == stringType: return resReq       # erreur SQL
        users =[]
        for d in resReq:        # resReq est une liste de tuples
            users.append('<OPTION VALUE=%s> %s %s </OPTION>' %\
                   (d[0], d[1], d[2]))
        users ="".join(users)       # conversion en chaîne
        
        
        # ************ Page provisoire pour tests graphiques **********************
        
        #histogrammePollens(200, 1)
        #map =paysageVirtuel(200)
        #return '<h3>Paysage reconstitué :</h3>'\
               #'<img class=spaced src="/static/histogr_polli0.png">'\
               #'<img class=spaced src="/static/paysage0.png">'
    
        # *************************************************************************
        
        return ttt("com", "userPrompt") %\
        (msg, _("Veuillez SVP vous identifier dans la liste :"),
         users, _("Votre mot de passe :"), _("Entrer un nouvel utilisateur"))

    userPrompt.exposed =True    

    def inputUser(self):
        # Encodage d'un nouvel utilisateur
        return ttt("com", "inputUser") %\
        (_("Veuillez vous identifier, SVP :"),
         _("Votre nom :"), _("Votre prénom :"),
         _("Choisissez un mot de passe :"),
         _("Confirmation du mot de passe :"))
 
    inputUser.exposed =True    

    def greetUser(self, nom =None, prenom =None, passwd=None, lp =None):
        # accueil de l'utilisateur qui vient de se connecter
        host = cherrypy.request.headers['Host']
        jc =True        # just connected
        if not nom or not prenom:
            # s'agit-il de quelqu'un qui continue sa session ?
            nom = cherrypy.session.get('nom', None)
            prenom = cherrypy.session.get('prenom', None)
            jc =False
        if not nom or not prenom:
            # N'a pas fourni son nom ou son prénom :
            return '<H4>%s</BR></BR><A HREF="/start/userPrompt">%s</a></H4>'%\
            (_("Veuillez SVP choisir un nom et un prénom."),
            _("Retour à la page d'accueil"))
        # -------- L'accès a donc été autorisé : ----------------
        # RAZ dico de session :
        nom, prenom = nom.strip(), prenom.strip()
        cherrypy.session["memPol"] =None      # champ micro. absent
        cherrypy.session["region"] =None      # région de prospection
        cherrypy.session["detPol"] =[]        # taxons déjà déterminés
        cherrypy.session["okTourb"] =0        # biotope tourbière choisi
        cherrypy.session["chPol_XY"] =None    # pas de cercle rouge !
        cherrypy.session['nom'] =nom
        cherrypy.session['prenom'] =prenom
        cherrypy.session["okTourb"] =0
        # Créer unrépertoire spécifique pour cet utilisateur :
        workDir ="./static/user%s" % (cherrypy.session["ref_usr"])
        if not os.path.exists(workDir):
            os.mkdir(workDir)
        # Retrouver les profondeurs déjà prospectées :
        tt =profondeursProspectees()
        if tt:                                # tuples (profondeur, datation)
            cherrypy.session["profond"] =tt[0][0]        
        # Mémoriser le login dans un fichier journal :
        if jc:          # just connected
            of =open(Glob.fichJour, "a")
            of.write("%s;%s;host=%s;%s\n" % (nom, prenom, host, now()))
            of.close()
        
        return '<a class="noUnderL" href="/notDec/choixRegion">'\
        '<h1 class="grisF">%s, %s %s.</h1><p>%s</p>'\
        '<h2 class="rougeMarge">%s</h2></a>' %\
        (_("Bonjour"), prenom, nom, ttt("msg","greetUser"),
        _("Bonne exploration !"))
        
    greetUser.exposed = True
    
    def mainMap(self):
        # Choix de la région à explorer.
        # Affichage de la carte principale (Europe), avec mise en place
        # d'une série de fenêtres popup contenant chacune une photo de la
        # région et un titre descriptif :
        popups =""
        for key, i in self.liPop.items():
            popups += '''<DIV ID="%sPop" CLASS="PopUp">
            <IMG SRC="/static/%s.jpg"></BR>%s</DIV>
            ''' % (key, i[0], i[1])
        
        return '''<div id="workSpace"> %s <LABEL CLASS=msg> %s </LABEL>
        %s </div>''' % (popups,
        _("Veuillez choisir la région de prospection sur la carte :"),
        ttt("com", "mainMap"))
        
    mainMap.exposed =True
    
    def generalHelp(self):
        return ttt("msg", "generalHelp")

    generalHelp.exposed =True

    def region(self):
        # L'utilisateur a choisi une région de prospection en Europe.
        # Brève présentation de cette région avant de choisir un biotope :        
        key = cherrypy.session.get("region", "eu")
        tReg =self.liPop[key]
        titre ='<H3>%s</H3><H2>%s</H2>' % (_("Région de prospection :"), tReg[1])
        page =''
        photos =tReg[2]        # noms des fichiers contenant les photos
        for p in photos:
            page +='<IMG SRC="/static/%s.jpg" alt="%s">' % (p, p)
        return '%s %s %s' % (titre, ttt("msg", "region"), page)

    region.exposed =True

    def biotope(self):
        # Choix des différents biotopes pour la recherche de pollens fossiles
        # (image map avec affichages divers en fonction des actions souris)

        # Mise en place d'une série de fenêtres popup :
        # - une mini contenant seulement une bulle d'aide (nom du biotope)
        # - une plus grande contenant :
        # * un commentaire
        # * un lien cliquable pointant vers la doc correspondante
        # * dans le cas de la tourbière, un second lien vers la suite du progr.
        # *** TO DO : Dans le cas du lac, le message devra varier en fonction
        #             de la région choisie ****************************
        # 
        diBio ={
        "lac":(_("Lac"), Glob.docPath, ttt("msg", "biotoLac"),
          "3,435,121,428,163,468,304,467,298,499,149,507,86,498,3,518"),
        "car":(_("Carrière avec dépôt de limon"), Glob.docPath,
          ttt("msg", "biotoCar"),
          "256,165,304,116,355,91,391,136,437,143,396,191"),
        "roc":(_("Rochers"), Glob.docPath, ttt("msg", "biotoRoc"),
          "463,122,511,195,558,171,617,237,526,224,437,239,397,215"),
        "fo1":(_("Forêt naturelle feuillue"), Glob.docPath,
          ttt("msg", "biotoFo1"),
          "243,311,274,262,345,250,348,201,437,252,392,282,392,355"),
        "fo2":(_("Forêt naturelle mixte (feuillus-résineux)"),
          Glob.docPath, ttt("msg", "biotoFo1"),
          "4,100,100,62,186,55,205,91,336,89,302,112,213,117,97,166"),
        "fo3":(_("Forêt plantée (résineux)"), Glob.docPath,
          ttt("msg","badBioto"),
          "703,378,679,438,763,469,839,417,805,388"),
        "tou":(_("Tourbière"), Glob.docPath,
          "<b>%s</b><br>" % (_("Excellent choix !")),
          "628,104,693,61,792,50,837,105,768,151",
          _("Prospecter (effectuer un carottage)")),
        "urb":(_("Ville"), Glob.docPath,
          ttt("msg","badBioto"),
          "346,580,399,452,601,407,711,578"),
        "vil":(_("Village"), Glob.docPath,
          ttt("msg","badBioto"),
          "598,201,638,259,742,256,810,200,743,178,683,205"),
        "ver":(_("Verger"), Glob.docPath,
          ttt("msg","badBioto"),
          "7,529,88,503,196,517,225,583,6,584"),
        "pra":(_("Prairie pâturée"), Glob.docPath,
          ttt("msg","badBioto"),
          "245,342,412,370,457,256,541,248,614,327,552,369,288,447,221,406"),
        "cul":(_("Cultures diverses"), Glob.docPath,
          ttt("msg","badBioto"),
          "5,4,5,95,99,51,186,49,220,89,367,85,394,126,438,136,464,106,514,186,558,158,579,180,647,45,513,2"),
        "ind":(_("Zone industrielle"), Glob.docPath,
          ttt("msg","badBioto"),
          "23,201,138,232,126,288,159,350,10,342"),
            }
        popups =""
        map ='<map name="map">'
        for key, it in diBio.items():
            # fenêtre popup avec commentaire :
            fm ='</BR><strong><a href="%s">%s</a></strong>' %\
            (it[1], _("En savoir plus"))
            if len(it) >4:   # cas de la tourbière => lien vers 'prospection' :
                fm +='</BR><strong><A HREF="/start/tourbiere">%s</a></strong>'\
                % (it[4])
            popups += '<DIV ID="%sBigPop" CLASS="bigPopUp" '\
            '''onclick="hidePopup('%s', true)">%s %s</DIV>\n''' %\
            (key, key, it[2], fm)
            map += '\n<area shape="poly" coords="%s"\n'\
            'href="javascript:nothing()" '\
            '''onclick="showBigPopup(event, '%s')" title="%s" />''' %\
            (it[3], key, it[0])
        
        map +="\n</map>\n"
        return '%s<p>%s</p> %s%s <p>%s</p>' %\
        (popups, ttt("msg", "biotoInstruct"), ttt("com", "biotopeMap"), map,
        _("Note : Les fenêtres documentaires peuvent être refermées "\
          "d'un simple clic."))
                                                       # (popups,
    biotope.exposed =True
    
    def tourbiere(self):
        # L'utilisateur a choisi de prospecter le biotope <tourbière>
        cherrypy.session["okTourb"] =1
        # On va lui poser une question de rélexion, puis continuer.
        # Rechercher si une réponse avait déjà été fournie :
        rep = trouveReponse(1)         # liste de 1 tuple dans ce cas
        answer, rAns, var =rep[0]

        if rAns:
            cherrypy.session["okTourb"] =3
            
            #cherrypy.session["okTourb"] =2
        #if len(answer) >5:
            #cherrypy.session["okTourb"] =3
        
        return ttt("com", "tourbiere") %\
        (ttt("quest", "1"),
         answer,
         _("Mémoriser la réponse"),
         rAns,
         ttt("msg","directivesQuest"),
         Glob.docPath, _("Consulter la documentation :"),
         _("Quels sont les dépôts contenant des spores et pollens fossiles ?"),
         _("Comment se forment les tourbières ?"),
         _("Comment prélever des sédiments meubles contenant des spores et pollens fossiles ?"))

    tourbiere.exposed =True

    def carottes(self, profond =1000000):
        profProsp =profondeursProspectees()
        choosed =''
        profond =int(profond)
        if profond <1000000:
            # l'utilisateur vient d'effectuer un prélèvement
            choosed ="<div class=left><p>" +\
            _("Voici votre échantillon de tourbe :") +"<br>" +\
            _("Vous devriez pouvoir y trouver des pollens fossiles,<br>"\
              "ou l'envoyer à un laboratoire spécialisé pour en obtenir "\
              "une datation au C14.") +"</p></div>"\
            '<img src="/static/ech_tourbe.gif" alt="échant. de tourbe" />'
            # (Ré)initialisations avant observation des grains :
            cherrypy.session["totalDet"] = []
            cherrypy.session["memPol"] =None
            cherrypy.session["detPol"] =[]        # taxons déjà déterminés
            cherrypy.session["profond"] =profond
            cherrypy.session["detErr"] =0         # erreurs de détermination
            cherrypy.session["okLabo"] =None      # prépa. micro. inexistante
            #cherrypy.session["okTourb"] =1

        # Préparer l'image des carottes avec ses zones cliquables :
        map = prepImageCarottes(profProsp, profond)
        return ttt("msg", "carottes") +\
        ttt("com","presCarottes") % (map, choosed)

    carottes.exposed =True

    def bugs(self):
        return ttt("msg", "bugs") 

    bugs.exposed =True

    def aPropos(self):
        return ttt("msg", "aPropos")

    aPropos.exposed =True

    def choisissezRegion(self):
        return "<br /><h4>"+\
        _("Vous devez d'abord choisir une région de prospection.") +'</h4>'
    
    choisissezRegion.exposed =True

    def choisissezEchantillon(self):
        return "<br /><h4>" +\
        _("Vous devez d'abord choisir un échantillon de tourbe.") +'</h4>'
    
    choisissezEchantillon.exposed =True
    
    def datationC14(self):
        # L'utilisateur a cliqué sur l'outil datation C14
        p = cherrypy.session.get("profond", 1000000)
        if p >999999:      # Aucune profondeur n'a encore été choisie
            return ttt("msg","datation_1")
        else:
            return '<br>%s<h3><a href="/start/dosageC14">%s</a> &nbsp; '\
            '<a href="/start/carottes">%s</a></h3><br>'\
            '<a href="%s"><h3>%s</h3><p>%s</p></a>' %\
            (ttt("msg","datation_2") % (p+5),
             _("oui"), _("non"), Glob.docPath,
             _("Consulter la documentation :"),
             ttt("msg","datation_5"))

    datationC14.exposed =True
    
    def dosageC14(self):
        rUsr =cherrypy.session["ref_usr"]
        # Vérification : maximum 3 datations par utilisateur :
        qy ="SELECT COUNT(r_pro) FROM depths WHERE dated >0 AND r_usr =%s" %\
           rUsr
        res = resRequeteBD(qy) 
        if isinstance(res, str): return res       # erreur SQL
        nb =res[0][0]
        if nb >=3:
            return "<h4>%s</h4>" % _("Vous avez déjà demandé 3 datations.")
        
        depth = cherrypy.session["profond"]
        date, dt =Glob.diProfDat[depth]         # Tuple date, incertitude

        # Ajouter ici une mémorisation de la datation effectuée (3 max)        
        # Mémorisation :
        qy ="SELECT r_pro FROM depths WHERE r_usr =%s AND depth =%s" %\
        (rUsr, depth)
        res = resRequeteBD(qy) 
        if isinstance(res, str): return res       # erreur SQL
        if len(res):
            rPro =res[0][0]
            qy ="UPDATE depths SET dated=%s WHERE r_pro =%s" % (date, rPro)
        else:
            n = enregSuivant("depths", "r_pro")
            qy ="INSERT INTO depths (r_pro, r_usr, depth, checked, errors, "\
            "dated, biome) VALUES (%s, %s, %s, %s, %s, %s, '%s')" %\
            (n, rUsr, depth, 0, 0, date, "")
        res =exeRequeteBD(qy)
        if res: return res          # error msg
        
        return ttt("msg","datation_3") % (depth +5, date, dt)
        
    dosageC14.exposed =True

    def interpretPaysa(self):
        # Synthèse du travail : on montre les paysages correspondant à
        # N profondeurs prospectées, suffisamment différentes.
        # On rappelle aussi les questions posées + réponses fournies.
        errMsg =_("Il vous faut d'abord prospecter !")
        if not cherrypy.session.get("ref_usr", None):
            return "<h3>%s</h3>" % errMsg 
        # Retrouver la liste des profondeurs déjà prospectées :
        profProsp =profondeursProspectees()
        if not isinstance(profProsp, list):
            return "<h3>%s</h3>" % errMsg 
        # Répertoire spécifique de l'utilisateur courant :
        workDir ="/static/user%s" % (cherrypy.session["ref_usr"])
        # Ébauche de page, à la fois dans une liste (destinée au rapport
        # imprimé), et dans une chaîne html :
        t1 =_("Principaux paysages reconstitués au cours de votre travail.")
        t2 =_("- À gauche, les principaux taxons présents.")
        t3 =_("- À droite, l'affinité climatique de ces taxons.")
        p2print =["<b>%s</b>" % t1, t2, t3]
        page = '<form action="/start/synthese" method=post><h3>%s %s</h3>%s' %\
        (_("Synthèse de votre travail :"),
         _("estimation de l'évolution climatique à partir de la végétation"),
         ttt("msg", "interpretPaysa"))
        # liste des choix de biomes proposés :
        tList =('-','*','1','1-11','2','3','4','5','6','7','8','9','10','11','12')
        # On n'affichera qu'un exemplaire des profondeurs "similaires"
        groups =((370,360),(350,340,330,320),(310,300),
                 (290,280,270,260,250,240,230,220),
                 (210,200,190,180,170,160,150,140,130,120,110),
                 (100,90,80,70,60,50,40),(30,20,10),(5,))

        # Présentation d'une sélection de paysages (versions N et S) :
        profAff =list()        # liste des profondeurs déjà affichées
        for prof, dat, biome in profProsp:
            # faut-il omettre cette profondeur ? 
            br =False
            for pA in profAff:
                for gr in groups:
                    #print "prof %s pAff %s grp %s" % (prof, pA, gr)
                    if prof in gr and pA in gr:
                        br =True ; break
            if br:
                pp ="<strong>%s -%s cm : %s.</strong>" % (_("Profondeur"),
                prof+5, _("situation similaire à celle décrite ci-dessus"))
                page +="&nbsp; %s<br>\n" % pp
                p2print.append(pp)
                continue
            page +="<HR>"       # ligne de séparation
            # Biomes à choisir, proposés dans une boîte de liste :
            oList =[]
            for i in tList:
                if i ==biome: oo=" selected"
                else: oo =""
                oList.append("<option%s>%s</option>" % (oo, i))
            oList ="".join(oList)
            # Sélection des images construites au cours du travail
            fileN ="%s/paysa_n_%s.png" % (workDir, prof)
            if os.path.exists(".%s" % (fileN)):
                fileC ="%s/paysa_c_%s.png" % (workDir, prof)
                # *** On ne souhaite plus afficher les datations ici : ***
                #if dat:
                    #dat =Glob.diProfDat[prof]
                    #c14 = ". %s = %s cal BP (Before Present) :" %\
                    #(_("Datation (C14)"), dat)
                #else: c14 =":"
                c14 =":"
                page +='<p><b>%s = -%s cm %s</b></p>\n<table>'\
                '<tr><td><img src="%s" class="spaced" ALT="paysage" /></td>'\
                '<td><img src="%s" ALT="paysage" /></td>\n'\
                '<td><img src="/static/PFT_noms_mini.jpg" /><br><br>'\
                '<b>%s</b><br><br><select name="sel%s">%s</select>'\
                '</td></tr>\n</table>\n' %\
                (_("Profondeur"), prof+5, c14, fileN, fileC,
                 _("Choisissez le groupe dominant probable :"), prof, oList)
                p2print.append("<b>%s = -%s cm %s</b>" %\
                               (_("Profondeur"), prof+5, c14))
                oo ="%s : %s" % (_("Groupe dominant probable"), biome)
                p2print.append("*leg*%s" % oo)
                p2print.append("*ima*%s*%s" % (fileN, fileC))
                profAff.append(prof)     # Mémoriser la profondeur affichée

                # Ajout des questions & réponses déjà fournies :
                for qtp in Glob.listeQuesPays:      # parcourir les questions
                    liPro =qtp[0]           # profondeurs concernées par la q.
                    #if prof not in liPro: continue
                    if liPro[0] >=0 and prof not in liPro: continue
                    numQ =qtp[2]            # N° de la question
                    # ne rappeler que les questions sur l'ensemble du paysage :
                    if numQ <12: continue
                    qq =trouveQuestion(numQ)  # liste de tuples (q,ans,rAQ,var)
                    for tu in qq:
                        quest, answ, rAQ, variable = tu
                        if "%s" in quest:            # question avec variable
                            quest =quest % (prof+5)
                            if variable ==str(prof+5):   # prof déjà rencontrée                      
                                break
                            else:
                                # réponse déjà fournie, mais pour autre profondeur
                                answ =""
                    page +="<p><strong>Q: </strong>%s<br>\n"\
                    "<strong>R: </strong>%s<br></p>\n" % (quest, answ)
                    p2print.append("<b>Q : </b>%s" % quest)
                    p2print.append("<b>R : </b>%s" % answ)

        page +='<HR><a href="%s"><h3>%s</h3><p>%s</p></a>'\
        '%s<img src="%s" class="spaced" ALT="Climatypes" />'\
        '<input type="submit" class="button" value="%s">' %\
        (Glob.docPath,
         _("Consulter la documentation :"),
         _("Quel est le lien entre la végétation et le climat ?"),
         ttt("msg", "interpretPaysa2"),
         Glob.pftFile,
         _("Valider vos entrées"))
        cherrypy.session["p2print"] =p2print
        return page

    interpretPaysa.exposed =True

    def synthese(self, **kwArgs):
        # Mémorisation des biomes choisis par l'utilisateur :
        rUsr =cherrypy.session["ref_usr"]
        bm =[]           # pour mémorisation temporaire des biomes choisis
        for k, v in kwArgs.items():
            if k[:3] =='sel':
                prof = int(k[3:])
                biome =v
                qy ="UPDATE depths SET biome='%s' WHERE r_usr=%s "\
                "AND depth=%s" % (biome, rUsr, prof)
                res =exeRequeteBD(qy)
                if res: return res               # error msg
                bm.append((prof,biome))
        
        # Il faut les mémoriser aussi dans la variable de session => PDF :
        bm.sort()               # trier dans l'ordre des profondeurs !!
        data =cherrypy.session["p2print"]
        i =0
        for n,line in enumerate(data):
            if line[:5] =="*leg*":
                # remplacement du n° de biome choisi :
                ll =line.split(":")
                ll[1] =" %s" % bm[i][1]
                line =":".join(ll)
                data[n] =line
                i +=1
        cherrypy.session["p2print"] =data

        return '<h4><br>%s<br><br>%s<br><br><a href ="/notDec/prospection">%s</a>'\
        '<br><br><a href="/start/conclusion"> %s </a></h4>' %\
        (_("OK. Vos entrées ont été mémorisées."), ttt("msg","conclusion"),
         _("Continuer à prospecter"),
         _("Conclure (imprimer le rapport final)"))

    synthese.exposed =True

    def conclusion(self):
        # Créer un document PDF reprenant la synthèse
        rUsr =cherrypy.session["ref_usr"]
        # Instanciation d'un objet 'Générateur de PDF' :
        PDF =PdfGenerator()
        fichier ="/static/user%s/palyno.pdf" % (rUsr)
        #os.remove(".%s" % fichier)
        can = Canvas(".%s" % (fichier), pagesize=A4)     # canevas Reportlab
        # Construction du document PDF :
        res = PDF.synthese(can)
        # sauvegarde du résultat dans le fichier PDF :
        can.save()
        # Page HTML fournissant le lien à télécharger :
        warn =""
        t1 = ttt("msg","pdfOK1")
        t2 = _("Veuillez cliquer ici pour y accéder")
        return '<br><H3><p>%s</p><p><A HREF="%s">%s</A></p></H3>' %\
               (t1, fichier, t2)

    conclusion.exposed =True

    def entrezUneReponse(self):
        return "<br /><h4>"+\
        _("Veuillez fournir une réponse, s.v.p. (5 car. minimum)") +\
        '<br /><a href="/start/tourbiere"><br />' +\
        _("Retour") +'</a></h4>'

    entrezUneReponse.exposed =True
    
    def questionnaire(self):
        rUsr =cherrypy.session.get("ref_usr", None)
        if not rUsr:
            return "<h3>%s</h3>" % (_("Vous devez d'abord vous identifier."))
        # Le texte des questions est extrait du fichier palyno_quest.txt :
        dq =dicoQuestions()
        # Recherche des réponses déjà fournies :
        qy ="SELECT r_que, answer, variable FROM answers "\
        "WHERE r_usr=%s" % rUsr
        res = resRequeteBD(qy) 
        if isinstance(res, str): return res       # erreur SQL
        lq =[]
        for num, quest in dq.items():
            # Réponse(s) éventuelles à cette question :
            rep =""
            # Ajouter seulement les questions déjà rencontrées :
            for answ in res:
                if answ[0] ==num:
                    rep, var = answ[1], answ[2]
                    if num >16: break
                    # Texte de la question :
                    if "%s" in quest: qq =quest % var
                    else: qq =quest; var ="§"
                    lq.append('<OPTION VALUE="%s#%s#%s">%s... | %s...</OPTION>' %\
                    (num, var, qq, qq[:90], rep[:15]))
            if num>16:   # ajouter aussi les questions de synthèse
                lq.append('<OPTION VALUE="%s#§#%s">%s... | %s...</OPTION>' %\
                (num, quest, quest[:90], rep[:15]))
        options = "".join(lq)
        return ttt("com", "questionnaire2") %\
        (ttt("msg","questionnaire"),
         options,
         _("Entrer ou modifier la réponse pour la question sélectionnée"))  

    questionnaire.exposed =True
    
class PdfGenerator:
    # Générateur de pages PDF. Aucune des méthodes ci-dessous n'est exposée.
    def __init__(self):
        self.largeurP, self.hauteurP = A4
        styles = getSampleStyleSheet()
        self.styleB =styles["BodyText"]
        self.styleG =styles["Heading3"]
        self.styleF =styles["Heading2"]
        self.styleH =styles["Heading1"]
        self.styleN =styles["Normal"]
        # rappel : les coord. Y se calculent à partir du bas de la page
        self.margeG, self.margeD = 1.5*cm, self.largeurP -1.5*cm
        self.margeS, self.margeB = self.hauteurP -1.5*cm, 1.5*cm


    def synthese(self, c):
        # ========= Pour retrouver les polices disponibles : =======
        #print c.getAvailableFonts()
        # ==========================================================
        
        def dimImage(diaFile, sx):
            # redimensionner l'image diaFile à la largeur sx
            im =Image.open(diaFile)
            sizeX, sizeY =im.size
            ratio =1.*sizeY/sizeX
            sizeX =sx*cm                  # Dimension X imposée
            sizeY =sizeX *ratio
            return (sizeX, sizeY)

        def printParagr(txt):
            p = Paragraph(convUtf8(txt), myStyle)
            l, h = p.wrap(lDisp, hDisp)      # calcul esp. nécessaire
            p.drawOn(c, self.margeG, vPos-h +12)
            #vPos -= (h +3)
            return vPos -h-3


        pdfmetrics.registerFont(TTFont('Times', 'Times_New_Roman.ttf'))
        pdfmetrics.registerFont(TTFont('TimesBI', 'Times_New_Roman_Bold_Italic.ttf'))
        pdfmetrics.registerFont(TTFont('TimesI', 'Times_New_Roman_Italic.ttf'))
        pdfmetrics.registerFont(TTFont('TimesB', 'Times_New_Roman_Bold.ttf'))

        d =DateTime.now()
        nPage =1
        vPos =self.margeS
        tt =_("Rapport sur la simulation effectuée à l'aide du logiciel Palyno")
        c.setFont("TimesBI", 15)
        c.drawCentredString(self.largeurP/2, vPos, convUtf8(tt))
        vPos -=22
        nom =cherrypy.session["nom"]
        prenom =cherrypy.session["prenom"]
        rUsr =cherrypy.session["ref_usr"]
        tt ="%s : %s %s" % (_("Utilisateur"), prenom, nom)
        tt ="%s - Date : %02d/%02d/%s" % (tt, d.day, d.month, d.year)
        c.setFont("TimesB", 12)
        c.drawString(self.margeG, vPos, convUtf8(tt))
        vPos -=18
        
        # Copie locale d'un style standard pour l'adapter au cas présent : 
        myStyle =deepcopy(self.styleN)
        myStyle.fontSize =12
        myStyle.leading =14         # interligne
        lDisp, hDisp = 500, 300     # larg. et haut. disponibles pour paragr.
        # Petit texte d'intro :
        vPos =printParagr(ttt("msg", "synthesePDF1"))
        # === Insertion du diagramme pollinique : ===
        diaFile ="./static/user%s/diagr_polli.png" % rUsr
        # Supprimer les boutons 'paysage' :
        imo =Image.open(diaFile)
        sizeX, sizeY =imo.size
        if sizeX >850:
            sizeX -=82
            imr =imo.crop((0, 0, sizeX, sizeY))
            imr.save(diaFile)
        # Redimensionnement de l'image :
        sizeX, sizeY = dimImage(diaFile, 12)     # 12 cm = dimension X imposée
        c.drawImage(diaFile, self.margeG +50, vPos-sizeY+5,
                    width=sizeX*1.1, height =sizeY, mask =None)
        vPos -= (sizeY +12)
        # Rappel sur les datations C14 :
        c.setFont("Times", 12)
        txt =ttt("msg", "datation_4")[:-1]
        c.drawString(self.margeG+40, vPos+5, convUtf8(txt))
        vPos -=25
    
        # === Rappel du tableau des biomes : ===
        c.setFont("TimesB", 12)
        txt =\
        _("Principaux groupes fonctionnels de plantes, indicateurs du climat :")
        c.drawString(self.margeG, vPos, convUtf8(txt))
        vPos -=15
        diaFile =".%s" % Glob.pftFile
        # Redimensionnement de l'image :
        sizeX, sizeY = dimImage(diaFile, 13)
        c.drawImage(diaFile, self.margeG +50,
                    vPos-sizeY+5, width=sizeX, height=sizeY, mask=None)
        vPos -= (sizeY +12)
        
        # === Interprétation des paysages : ===
        for data in cherrypy.session["p2print"]:
            if data[:5] =="*ima*" :    
                # traitement des images de paysages :
                diaFile =data[5:].split("*")
                pay1 = ".%s" % diaFile[0]
                pay2 = ".%s" % diaFile[1]
                # Redimensionnement de l'image de gauche :
                sizeX, sizeY = dimImage(pay1, 8)
                c.drawImage(pay1, self.margeG+10, vPos-sizeY+5,
                            width=sizeX, height =sizeY, mask =None)
                # Redimensionnement de l'image de droite :
                sizeX, sizeY = dimImage(pay2, 8)
                c.drawImage(pay2, self.margeG+sizeX+20, vPos-sizeY+5,
                            width=sizeX, height =sizeY, mask =None)
                ## ajout du diagramme miniature des biomes :
                #c.drawImage("./static/PFT_noms_mini.jpg", self.margeG +11*cm,
                            #vPos-70, width=100, height=50, mask=None)
                vPos -= (sizeY +12)
            elif data[:5] =="*leg*" :
                # traitement d'une légende à droite :
                p = Paragraph(convUtf8(data[5:]), myStyle)
                l, h = p.wrap(220, 100)      # calcul esp. nécessaire
                p.drawOn(c, self.margeG +10*cm, vPos+12)
            else:
                # traitement d'un paragraphe :
                if vPos <130:
                    # Passage éventuel à la page suivante :
                    c.showPage()
                    vPos =self.margeS
                # Paragraphe :
                vPos =printParagr(data)
                #vPos =printParagr("<b>Q : </b>%s" % data)

                #p = Paragraph(convUtf8(data), myStyle)
                #l, h = p.wrap(lDisp, hDisp)      # calcul esp. nécessaire
                #p.drawOn(c, self.margeG, vPos-h +12)
                #vPos -= (h +3)

        # === Questions de synthèse : ===
        c.showPage()
        vPos =self.margeS
        c.setFont("TimesB", 12)
        c.drawString(self.margeG, vPos, convUtf8(_("Questions de synthèse")))
        vPos -=20
         # Le texte des questions est extrait du fichier palyno_quest.txt :
        dq =dicoQuestions()
        # Recherche des réponses déjà fournies :
        qy ="SELECT r_que, answer FROM answers WHERE r_usr=%s" % rUsr
        res = resRequeteBD(qy)
        if isinstance(res, str): return res       # erreur SQL
        for num, quest in dq.items():
            if num not in (1,2,17,18,19,20,21,22,23,24,25): continue
            rep =""
            for answ in res:
                if answ[0] ==num: rep =answ[1]
            # Paragraphe :
            if vPos <130:
                # Passage éventuel à la page suivante :
                c.showPage()
                vPos =self.margeS
            vPos =printParagr("<b>Q : </b>%s" % quest)
            vPos =printParagr("<b>R : </b>%s" % rep)


#########################################################
################ Programme principal ####################
#########################################################

# S'assurer que le répertoire de travail soit bien celui du présent script :
repDes =os.path.dirname(sys.argv[0])
if not os.path.isabs(repDes):
    repDes = os.getcwd() +"/" + repDes
os.chdir(repDes)

# Charger les données d'initialisation:
initialisation()

#if Glob.lang =="fr":
    #def _(chaine):
        #return chaine

# Procédures utilisées seulement au cours du développement : 
# Conversion du fichier CSV contenant le tableau des erreurs de détermination 
# Taxon fourni/Taxon correct => Indice menant au commentaire de correction :
# convertirCorrectifCSV()
# Procédure utilisée seulement au cours du développement : 
# Conversion du fichier CSV contenant le tableau des proportions des # pollens
# en fonction de la profondeur : 
# convertirComptageHF_CSV()
# Conversion du fichier CSV contenant le tableau des n.de symboles de taxons à
# inclure dans le paysage reconstitué pour chaque profondeur (Hautes-Fagnes) :
#convertirPaysageD_CSV()
#sys.exit()

print "*** Importing hint comments ..."
importerDiComCor()
print "*** Importing taxon data for each depth ..."
importerDiProfTax()
print "*** Importing C14 data for each depth ..."
importerDiProfDat()
print "*** Importing landscape building data ..."
importerDiLanDat()
print "*** Importing questions ..." 
importerListeQuesPays()
print "*** Opening users database : ...'",
# Ouverture de la base de données => création de l'interface BD :
ouvertureBD()
print "  O.K."

# ======= Debug & tests ====== : 
#for p in range(38):
#   histogrammePollens(p*10, var =0, nfi=p)
#diagrammePollinique((50,100,150,200,250,300,350,370))
#prepImageCarottes()
#sys.exit()
# ------------
#p =0
#histogrammePollens(p*10, 1)
#paysageVirtuel(p*10)
#for p in range(0,38,5):
    #paysageVirtuel(p*10, p)
#sys.exit()


print "*** Building pollen images library ..."
pollensEtCrasses()
print "*** Pollens and dirt : O.K."
champsMicroAgrandis(Glob.fieldSize2)
print "*** Zoomed microscopic fields : O.K."

# Au démarrage, CherryPy doit disposer d'un objet racine, produit à partir
# d'une classe dont les méthodes seront les gestionnaires de requête HTTP
# qui produiront les pages à afficher. Ainsi par exemple, une requête HTTP
# invoquant l'URL '/index.html' (ou plus simplement '/') sera traitée par
# le système comme étant une invocation de la méthode index() de cette classe :
cherrypy.root = PageAccueil()
print "*** Setting Cherrypy root page : O.K."
# Démarrage du serveur CherryPy, avec les options du fichier ad hoc :
cherrypy.config.update(file = "palyno.conf")    # config. de base
#'server.socket_port':Glob.httpPort,
cherrypy.config.update({'server.socket_port':Glob.httpPort,
                        'static.filter.root':repDes})
print "*** Starting server (lang = [%s]). Serving HTTP on port %s ..." %\
      (Glob.lang, Glob.httpPort) 
cherrypy.server.start()

########################################################################